/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * Authors: Zsolt Molnar <zsolt.molnar@cern.ch, http://www.zsoltmolnar.hu>
 *
 * Objects for dependency injection of SRM services.
 * (see: http://en.wikipedia.org/wiki/Dependency_injection)
 *
 * The goal: make the LCG_Util logic unit-testable by decoupling from the real Globus
 * calls. Unit tests then create mock objects and create response objects
 * without real Globus / network operations.
 */

/*
 * @(#)$RCSfile: globus_dependencies.h,v $ $Revision: 1.1 $ $Date: 2009/11/18 13:53:33 $ CERN Remi Mollon
 */
#ifndef LCG_GLOBUS_DEPENDENCIES_H__
#define LCG_GLOBUS_DEPENDENCIES_H__

#include "lcg_util.h"

/* enforce proper calling convention */
#ifdef __cplusplus
extern "C"
{
#endif

/* Function types of the appropriate SOAP calls */
typedef int (*lcg_globus_copyfilex_fv)(
    char *src_file, char *dest_file, int nbstreams, char *errbuf,
    int errbufsz, int timeout, int verbose);

typedef int (*lcg_globus_getfilesizet_fv)(
    char *file, GFAL_LONG64 *size, char *errbuf, int errbufsz, int timeout);

/* The collection of Globus calls. The pointers may be replaced with test
 * functions (dependency injection) */
typedef struct lcg_globus_callers {
    lcg_globus_copyfilex_fv call_globus_copyfilex;
    lcg_globus_getfilesizet_fv call_globus_getfilesizet;
} lcg_globus_callers_t;

extern lcg_globus_callers_t lcg_globus_functions;

#ifdef __cplusplus
}
#endif

#endif /* #ifndef LCG_GLOBUS_DEPENDENCIES_H__ */
