/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef lint
static char sccsid[] = "@(#)$RCSfile: lcg-bringonline.c,v $ $Revision: 1.1 $ $Date: 2009/04/08 13:05:11 $ CERN Remi Mollon";
#endif /* not lint */

/*	lcg-bringonline - bring surls on line */
#include <sys/types.h>
#include <errno.h>
#include <getopt.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "gfal_api.h"
#include "lcg_util.h"

int
main(int argc, char **argv)
{

	int helpflg = 0;
	int c;
	int errflg = 0;
	int verbose = 0;
	int version = 0;
	static struct option longopts[] = {
		{"file", required_argument, NULL, 'f'},
		{"st", required_argument, NULL, 's'},
		{"defaultsetype", required_argument, NULL, 'D'},
		{"setype", required_argument, NULL, 'T'},
		{"nobdii", no_argument, NULL, 'b'},
		{"protocols", required_argument, NULL, 'p'},
		{"timeout", required_argument, NULL, 't'},
		{"connect-timeout", required_argument, NULL, 130},
		{"sendreceive-timeout", required_argument, NULL, 131},
		{"bdii-timeout", required_argument, NULL, 132},
		{"srm-timeout", required_argument, NULL, 133},
		{"help", no_argument, NULL, 'h'},
		{"version", no_argument, NULL, 'W'},
		{"verbose", no_argument, NULL, 'v'},
		{0, 0, 0, 0}
	};
	gfal_request req;
	gfal_internal internal;
	gfal_filestatus *filestatuses;
	int timeout = 0, timeout_tmp = 0;
	char *file = NULL;
	char *dp, *p1, *p2;
	char **protocols = NULL, **surls = NULL;
	int i, n, rc;

	if ((req = gfal_request_new ()) == NULL) {
		perror ("lcg-bringonline");
		exit (1);
	}

	while ((c = getopt_long (argc, argv, "f:s:D:T:bp:t:hvW", longopts, NULL)) != EOF) {
		switch (c) {
			case 'f':
				file = optarg;
				break;
			case 's':
				req->srmv2_spacetokendesc = optarg;
				break;
			case 'D':
				if (!strcasecmp (optarg, "srmv1"))
					req->defaultsetype = TYPE_SRM;
				else if (!strcasecmp (optarg, "srmv2"))
					req->defaultsetype = TYPE_SRMv2;
				else if (!strcasecmp (optarg, "se"))
					req->defaultsetype = TYPE_SE;
				else {
					fprintf (stderr,"Invalid SE type: %s\n", optarg);
				}
				break;
			case 'T':
				if (!strcmp (optarg, "srmv1"))
					req->setype = TYPE_SRM;
				else if (!strcmp (optarg, "srmv2"))
					req->setype = TYPE_SRMv2;
				else if (!strcmp (optarg, "se"))
					req->setype = TYPE_SE;
				else {
					fprintf (stderr,"Invalid SE type: %s\n", optarg);
					errflg++;
				}
				break;
			case 'b':
				req->no_bdii_check = 1;
				break;
			case 'p':
                // TODO hardcoded 10, see the next comment...
				if ((protocols = (char **) calloc (10, sizeof (char *))) == NULL) {
					errno = ENOMEM;
					perror ("lcg-bringonline");
					exit (1);
				}
				p1 = optarg;
                // TODO This is obviously wrond, what if the protocol list exceeds 10? Crash.
				for (i = 0; (p2 = strchr (p1, ',')); ++i) {
					*p2 = 0;
					protocols[i] = p1;
					p1 = p2 + 1;
				}
				protocols[i++] = p1;
				protocols[i] = NULL;
				req->protocols = protocols;
				break;
			case 't':
				fprintf (stderr, "Warning: -t,--timeout is deprecated! Use --timeout-* options instead\n");
				if ((timeout = strtol (optarg, &dp, 10)) < 0 || 
						*dp != '\0' ) {
					fprintf (stderr,
							"invalid value for '--timeout': %s\n", optarg);
					errflg++;
				}
			case 130: // "connect-timeout"
				if ((timeout_tmp = strtol (optarg, &dp, 10)) < 0 || 
						*dp != '\0' ) {
					fprintf (stderr,
							"invalid value for '--connect-timeout': %s\n", optarg);
					errflg++;
				} else
					gfal_set_timeout_connect (timeout_tmp);
				break;
			case 131: // "sendreceive-timeout"
				if ((timeout_tmp = strtol (optarg, &dp, 10)) < 0 || 
						*dp != '\0' ) {
					fprintf (stderr,
							"invalid value for '--sendreceive-timeout': %s\n", optarg);
					errflg++;
				} else
					gfal_set_timeout_sendreceive (timeout_tmp);
				break;
			case 132: // "bdii-timeout"
				if ((timeout_tmp = strtol (optarg, &dp, 10)) < 0 || 
						*dp != '\0' ) {
					fprintf (stderr,
							"invalid value for '--bdii-timeout': %s\n", optarg);
					errflg++;
				} else
					gfal_set_timeout_bdii (timeout_tmp);
				break;
			case 133: // "srm-timeout"
				if ((timeout_tmp = strtol (optarg, &dp, 10)) < 0 || 
						*dp != '\0' ) {
					fprintf (stderr,
							"invalid value for '--srm-timeout': %s\n", optarg);
					errflg++;
				} else
					gfal_set_timeout_srm (timeout_tmp);
				break;
			case 'W':
				version = 1;
				break;
			case 'v':
				++verbose;
				break;
			case 'h':
				helpflg = 1;
				break;
			default:
				fprintf (stderr, "Unknown option %c\n", c);
				errflg++;
				break;
		}
	}

	if (version) {
		printf ("lcg_util-%s\n", lcg_util_version ());
		printf ("GFAL-client-%s\n", gfal_version ());
		exit (0);
	}
	if ((!file && optind >= argc) || (file && optind != argc))
		errflg++;

	if (errflg || helpflg) {
		free (req);
		fprintf (stderr, "usage: %s %s%s%s", argv[0],
				"[-h,--help] [-s,--st spacetokendesc] [-D,--defaultsetype se|srmv1|srmv2]] [-p protocol[,...]]\n",
				"\t[--connect-timeout value] [--sendreceive-timeout value] [--bdii-timeout value]\n",
				"\t[--srm-timeout value] [-T,--setype se|srmv1|srmv2] [-b,--nobdii] [--version] (-f file | surl [...])\n");
		exit (errflg ? 1 : 0);
	}

	gfal_set_verbose (verbose);

	if (file) {
		if ((req->nbfiles = import_surls_from_file (file, &surls, NULL, 0)) < 0) {
			perror ("lcg-bringonline");
			exit (1);
		}
		req->surls = surls;
	} else {
		req->surls = argv + optind;
		req->nbfiles = argc - optind;
	}

	if (check_surls (req->nbfiles, req->surls, NULL, 0) < 0) {
		perror ("lcg-bringonline");
		exit (1);
	}

	if (gfal_init (req, &internal, NULL, 0) < 0) {
		if (protocols) free (protocols);
		if (surls) free (surls);
		free (req);
		perror ("lcg-bringonline");
		exit (1);
	}
	free (req);

	if (verbose)
		printf ("SE type: %s\n", lcg_se_type_string (internal->setype));

	if (gfal_bringonline (internal, NULL, 0) < 0) {
		gfal_internal_free (internal);
		if (protocols) free (protocols);
		if (surls) free (surls);
		perror ("lcg-bringonline");
		exit (1);
	}

	if (verbose) {
		char *srmv2_reqtoken = NULL;
		int srmv1_reqid;

		if (gfal_get_ids (internal, &srmv1_reqid, NULL, &srmv2_reqtoken) < 0)
			printf ("SRM Request Token: [ERROR]\n");
		else if (srmv2_reqtoken)
			printf ("SRM Request Token: %s\n", srmv2_reqtoken);
		else
			printf ("SRM Request Token: %d\n", srmv1_reqid);
	}

	if ((n = gfal_get_results (internal, &filestatuses)) < 0) {
		if (protocols) free (protocols);
		if (surls) free (surls);
		gfal_internal_free (internal);
		fprintf (stderr, "lcg-bringonline: Cannot get results !\n");
		exit (1);
	}

	rc = 0;

	for (i = 0; i < n; ++i) {
		if (filestatuses[i].status) {
			++rc;
			if (filestatuses[i].explanation)
				fprintf (stderr, "%s: %s\n", filestatuses[i].surl, filestatuses[i].explanation);
			else
				fprintf (stderr, "%s: %s\n", filestatuses[i].surl, strerror (filestatuses[i].status));
		}
	}

	if (protocols) free (protocols);
	if (surls) free (surls);
	gfal_internal_free (internal);

	exit (rc);
}
