/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef lint
static char sccsid[] = "@(#)$RCSfile: lcg-get-checksum.c,v $ $Revision: 1.2 $ $Date: 2009/04/08 14:23:45 $ CERN Jean-Philippe Baud";
#endif /* not lint */

/*	lcg-get-checksum - get the TURL for a given SURL and transfer protocol */
#include <sys/types.h>
#include <errno.h>
#include <getopt.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <strings.h>
#include "lcg_util.h"

	int
main (int argc, char **argv)
{
	int rc = 0;
	int helpflg = 0;
	int verbose = 0;
	int version = 0;
	int force = 0;
	int c = 0, i = 0;
	int errflg = 0;
	int timeout = 0, timeout_tmp = 0;
	static struct option longopts[] = {
		{"help", no_argument, NULL, 'h'},
		{"file", required_argument, NULL, 'f'},
		{"force", no_argument, NULL, 'F'},
		{"checksum-type", required_argument, NULL, 'K'},
		{"defaultsetype", required_argument, NULL, 'D'},
		{"setype", required_argument, NULL, 'T'},
		{"nobdii", no_argument, NULL, 'b'},
		{"timeout", required_argument, NULL, 't'},
		{"connect-timeout", required_argument, NULL, 130},
		{"sendreceive-timeout", required_argument, NULL, 131},
		{"bdii-timeout", required_argument, NULL, 132},
		{"srm-timeout", required_argument, NULL, 133},
		{"verbose", no_argument, NULL, 'v'},
		{"vo", required_argument, NULL, 'V'},
		{"version", no_argument, NULL, 'W'},
		{0, 0, 0, 0}
	};
	char *vo = NULL;
	char *dp = NULL;
	enum se_type defaulttype = TYPE_NONE, setype = TYPE_NONE;
	int nobdii = 0;
	int nbsurls = 0;
	char *file = NULL, **surls = NULL;
	enum gfal_cksm_type cksmtype = GFAL_CKSM_NONE;
	lcg_checksum *checksums = NULL;

	while ((c = getopt_long (argc, argv, "hfFivV:t:s:K:D:T:bW", longopts, NULL)) != EOF) {
		switch (c) {
			case 'h':
				helpflg = 1;
				break;
			case 'f':
				file = optarg;
				break;
			case 'F':
				force = 1;
				break;
			case 't':
				fprintf (stderr, "Warning: -t,--timeout is deprecated! Use --timeout-* options instead\n");
				if ((timeout = strtol (optarg, &dp, 10)) < 0 || 
						*dp != '\0' ) {
					fprintf (stderr,
							"invalid value for '--timeout': %s\n", optarg);
					++errflg;
				}
			case 130: // "connect-timeout"
				if ((timeout_tmp = strtol (optarg, &dp, 10)) < 0 || 
						*dp != '\0' ) {
					fprintf (stderr,
							"invalid value for '--connect-timeout': %s\n", optarg);
					++errflg;
				} else
					gfal_set_timeout_connect (timeout_tmp);
				break;
			case 131: // "sendreceive-timeout"
				if ((timeout_tmp = strtol (optarg, &dp, 10)) < 0 || 
						*dp != '\0' ) {
					fprintf (stderr,
							"invalid value for '--sendreceive-timeout': %s\n", optarg);
					++errflg;
				} else
					gfal_set_timeout_sendreceive (timeout_tmp);
				break;
			case 132: // "bdii-timeout"
				if ((timeout_tmp = strtol (optarg, &dp, 10)) < 0 || 
						*dp != '\0' ) {
					fprintf (stderr,
							"invalid value for '--bdii-timeout': %s\n", optarg);
					++errflg;
				} else
					gfal_set_timeout_bdii (timeout_tmp);
				break;
			case 133: // "srm-timeout"
				if ((timeout_tmp = strtol (optarg, &dp, 10)) < 0 || 
						*dp != '\0' ) {
					fprintf (stderr,
							"invalid value for '--srm-timeout': %s\n", optarg);
					++errflg;
				} else
					gfal_set_timeout_srm (timeout_tmp);
				break;
			case 'K':
				cksmtype = gfal_parse_checksum_algname (optarg);
				if (cksmtype == GFAL_CKSM_NONE) {
					fprintf (stderr,"Invalid checksum type: %s\n", optarg);
				}
				break;
			case 'D':
				if (!strcasecmp (optarg, "srmv1"))
					defaulttype = TYPE_SRM;
				else if (!strcasecmp (optarg, "srmv2"))
					defaulttype = TYPE_SRMv2;
				else if (!strcasecmp (optarg, "se"))
					defaulttype = TYPE_SE;
				else {
					fprintf (stderr,"Invalid SE type: %s\n", optarg);
					++errflg;
				}
				break;
			case 'T':
				if (!strcmp (optarg, "srmv1"))
					setype = TYPE_SRM;
				else if (!strcmp (optarg, "srmv2"))
					setype = TYPE_SRMv2;
				else if (!strcmp (optarg, "se"))
					setype = TYPE_SE;
				else {
					fprintf (stderr,"Invalid SE type: %s\n", optarg);
					++errflg;
				}
				break;
			case 'b':
				nobdii = 1;
				break;
			case 'v':
				++verbose;
				break;
			case 'V':
				vo = optarg;
				break;
			case 'W':
				version = 1;
				break;
			case '?':
			default:
				++errflg;
				break;
		}
	}
	if (version) {
		printf ("lcg_util-%s\n", lcg_util_version ());
		printf ("GFAL-client-%s\n", gfal_version ());
		exit (0);
	}
	if ((!file && optind >= argc) || (file && optind != argc))
		++errflg;
	if (errflg || helpflg) {
		fprintf (stderr, "usage: %s %s%s%s%s", argv[0],
				"[-h,--help] [-b,--nobdii] [--srm-timeout value] [--connect-timeout value]\n",    
				"\t[--force] [--sendreceive-timeout value] [--bdii-timeout value]\n",
				"\t[-D,--defaultsetype se|srmv1|srmv2] [-T,--setype se|srmv1|srmv2]\n",
				"\t[--checksum-type type] [-v,--verbose] [--version] (-f file | surl [...])\n");
		exit (errflg ? 1 : 0);
	}

	gfal_set_verbose (verbose);

	if (file) {
		if ((nbsurls = import_surls_from_file (file, &surls, NULL, 0)) < 0) {
			perror ("lcg-get-checksum");
			exit (1);
		}
	} else {
		surls = argv + optind;
		nbsurls = argc - optind;
	}

	rc = lcg_get_checksum (nbsurls, surls, defaulttype, setype, nobdii, force, vo, cksmtype, &checksums, NULL, 0);

	if (checksums == NULL) {
		if (errno)
			perror ("lcg-get-checksum");
		exit (1);
	}

	for (i = 0; i < nbsurls; ++i) {
		if (checksums[i].status == 0)
			printf ("%s\t%s\n", checksums[i].checksum, checksums[i].file);
		else
			gfal_errmsg (NULL, 0, GFAL_ERRLEVEL_ERROR, "[ERROR] %s: %s", checksums[i].file,
					checksums[i].explanation ? checksums[i].explanation : strerror (checksums[i].status));

		if (checksums[i].file) free (checksums[i].file);
		if (checksums[i].checksum) free (checksums[i].checksum);
		if (checksums[i].explanation) free (checksums[i].explanation);
	}

	if (file) free (surls);
	free (checksums);
	exit (rc == 0 ? 0 : 1);
}
