/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef lint
static char sccsid[] = "@(#)$RCSfile: lcg-la.c,v $ $Revision: 1.14 $ $Date: 2008/11/21 15:19:48 $ CERN Jean-Philippe Baud";
#endif /* not lint */

/*	lcg-la - list the aliases for a given LFN, GUID or SURL */
#include <sys/types.h>
#include <errno.h>
#include <getopt.h>
#include <stdlib.h>
#include <stdio.h>
#include "lcg_util.h"

	int
main (int argc, char **argv)
{
	int helpflg = 0;
	int insecure = 0;
	int verbose = 0;
	int version = 0;
	int c;
	char *conf_file = NULL;
	int errflg = 0;
	int i;
	char **lfns;
	static struct option longopts[] = {
		{"config", required_argument, NULL, 'c'},
		{"help", no_argument, NULL, 'h'},
		{"insecure", no_argument, NULL, 'i'},
		{"verbose", no_argument, NULL, 'v'},
		{"nobdii", no_argument, NULL, 'b'},
		{"vo", required_argument, NULL, 'V'},
		{"connect-timeout", required_argument, NULL, 130},
		{"sendreceive-timeout", required_argument, NULL, 131},
		{"bdii-timeout", required_argument, NULL, 132},
		{"version", no_argument, NULL, 'W'},
		{0, 0, 0, 0}
	};
	char *vo = NULL;
	int nobdii = 0;
	int timeout_tmp = 0;
	char *dp = NULL;

	while ((c = getopt_long (argc, argv, "c:hibvV:W", longopts, NULL)) != EOF) {
		switch (c) {
			case 'c':
				conf_file = optarg;
				break;
			case 'h':
				helpflg = 1;
				break;
			case 'i':
				insecure = 1;
				break;
			case 'b':
				nobdii = 1;
				break;
			case 'v':
				++verbose;
				break;
			case 'V':
				vo = optarg;
				break;
			case 130: // "connect-timeout"
				if ((timeout_tmp = strtol (optarg, &dp, 10)) < 0 || 
						*dp != '\0' ) {
					fprintf (stderr,
							"invalid value for '--connect-timeout': %s\n", optarg);
					errflg++;
				} else
					gfal_set_timeout_connect (timeout_tmp);
				break;
			case 131: // "sendreceive-timeout"
				if ((timeout_tmp = strtol (optarg, &dp, 10)) < 0 || 
						*dp != '\0' ) {
					fprintf (stderr,
							"invalid value for '--sendreceive-timeout': %s\n", optarg);
					errflg++;
				} else
					gfal_set_timeout_sendreceive (timeout_tmp);
				break;
			case 132: // "bdii-timeout"
				if ((timeout_tmp = strtol (optarg, &dp, 10)) < 0 || 
						*dp != '\0' ) {
					fprintf (stderr,
							"invalid value for '--bdii-timeout': %s\n", optarg);
					errflg++;
				} else
					gfal_set_timeout_bdii (timeout_tmp);
				break;
			case 'W':
				version = 1;
				break;
			case '?':
				errflg++;
				break;
			default:
				break;
		}
	}
	if (version) {
		printf ("lcg_util-%s\n", lcg_util_version ());
		printf ("GFAL-client-%s\n", gfal_version ());
		exit (0);
	}
	if (optind + 1 != argc)
		errflg++;
	if (errflg || helpflg) {
		fprintf (stderr, "usage: %s %s%s", argv[0],
				"[--connect-timeout value] [--sendreceive-timeout value] [--bdii-timeout value]\n",
				"\t[-c,--config config_file] [-h,--help] [-V,--vo vo] [-b,--nobdii] [--version] file\n");
		exit (errflg ? 1 : 0);
	}

	if (lcg_la2 (argv[optind], nobdii, vo, &lfns, NULL, 0) < 0) {
		if (errno)
			perror ("lcg_la");
		exit (1);
	}
	i = 0;
	while (lfns[i]) {
		printf ("lfn:%s\n", lfns[i]);
		free (lfns[i++]);
	}
	free (lfns);
	exit (0);
}
