/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef lint
static char sccsid[] = "@(#)$RCSfile: lcg-rf.c,v $ $Revision: 1.13 $ $Date: 2008/11/21 15:19:48 $ CERN Jean-Philippe Baud";
#endif /* not lint */

/*	lcg-rf - registers in the LRC (and optionally in the RMC) a file
 *	residing on an SE
 */
#include <sys/types.h>
#include <errno.h>
#include <getopt.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <strings.h>
#include "lcg_util.h"

	int
main(int argc, char **argv)
{
	int helpflg = 0;
	int insecure = 0;
	int verbose = 0;
	int version = 0;
	char actual_guid[37];
	int c;
	char *conf_file = NULL;
	int errflg = 0;
	char *guid = NULL;
	char *lfn_arg = NULL;
	char *dp;
	static struct option longopts[] = {
		{"config", required_argument, NULL, 'c'},
		{"help", no_argument, NULL, 'h'},
		{"insecure", no_argument, NULL, 'i'},
		{"defaultsetype", required_argument, NULL, 'D'},
		{"setype", required_argument, NULL, 'T'},
		{"nobdii", no_argument, NULL, 'b'},
		{"timeout", required_argument, NULL, 't'},
		{"connect-timeout", required_argument, NULL, 130},
		{"sendreceive-timeout", required_argument, NULL, 131},
		{"bdii-timeout", required_argument, NULL, 132},
		{"srm-timeout", required_argument, NULL, 133},
		{"verbose", no_argument, NULL, 'v'},
		{"vo", required_argument, NULL, 'V'},
		{"version", no_argument, NULL, 'W'},
		{0, 0, 0, 0}
	};
	char *vo = NULL;
	int timeout = 0, timeout_tmp = 0;
	enum se_type defaulttype = TYPE_NONE, setype = TYPE_NONE;
	int nobdii = 0;

	while ((c = getopt_long (argc, argv, "bc:D:g:hil:vV:t:T:W", longopts, NULL)) != EOF) {
		switch (c) {
			case 'c':
				conf_file = optarg;
				break;
			case 'g':
				guid = optarg;
				break;
			case 'h':
				helpflg = 1;
				break;
			case 'i':
				insecure = 1;
				break;
			case 'l':
				lfn_arg = optarg;
				break;
			case 'V':
				vo = optarg;
				break;
			case 'D':
				if (!strcasecmp (optarg, "srmv1"))
					defaulttype = TYPE_SRM;
				else if (!strcasecmp (optarg, "srmv2"))
					defaulttype = TYPE_SRMv2;
				else if (!strcasecmp (optarg, "se"))
					defaulttype = TYPE_SE;
				else {
					fprintf (stderr,"Invalid SE type: %s\n", optarg);
				}
				break;
			case 'T':
				if (!strcmp (optarg, "srmv1"))
					setype = TYPE_SRM;
				else if (!strcmp (optarg, "srmv2"))
					setype = TYPE_SRMv2;
				else if (!strcmp (optarg, "se"))
					setype = TYPE_SE;
				else {
					fprintf (stderr,"Invalid SE type: %s\n", optarg);
					errflg++;
				}
				break;
			case 'b':
				nobdii = 1;
				break;
			case 'v':
				++verbose;
				break;
			case 't':
				fprintf (stderr, "Warning: -t,--timeout is deprecated! Use --timeout-* options instead\n");
				if ((timeout = strtol (optarg, &dp, 10)) < 0 || 
						*dp != '\0' ) {
					fprintf (stderr,
							"invalid value for '--timeout': %s\n", optarg);
					errflg++;
				}
			case 130: // "connect-timeout"
				if ((timeout_tmp = strtol (optarg, &dp, 10)) < 0 || 
						*dp != '\0' ) {
					fprintf (stderr,
							"invalid value for '--connect-timeout': %s\n", optarg);
					errflg++;
				} else
					gfal_set_timeout_connect (timeout_tmp);
				break;
			case 131: // "sendreceive-timeout"
				if ((timeout_tmp = strtol (optarg, &dp, 10)) < 0 || 
						*dp != '\0' ) {
					fprintf (stderr,
							"invalid value for '--sendreceive-timeout': %s\n", optarg);
					errflg++;
				} else
					gfal_set_timeout_sendreceive (timeout_tmp);
				break;
			case 132: // "bdii-timeout"
				if ((timeout_tmp = strtol (optarg, &dp, 10)) < 0 || 
						*dp != '\0' ) {
					fprintf (stderr,
							"invalid value for '--bdii-timeout': %s\n", optarg);
					errflg++;
				} else
					gfal_set_timeout_bdii (timeout_tmp);
				break;
			case 133: // "srm-timeout"
				if ((timeout_tmp = strtol (optarg, &dp, 10)) < 0 || 
						*dp != '\0' ) {
					fprintf (stderr,
							"invalid value for '--srm-timeout': %s\n", optarg);
					errflg++;
				} else
					gfal_set_timeout_srm (timeout_tmp);
				break;
			case 'W':
				version = 1;
				break;
			case '?':
				errflg++;
				break;
			default:
				break;
		}
	}
	if (version) {
		printf ("lcg_util-%s\n", lcg_util_version ());
		printf ("GFAL-client-%s\n", gfal_version ());
		exit (0);
	}
	if (optind + 1 != argc) {
		errflg++;
	}
	if (errflg || helpflg) {
		fprintf (stderr, "usage: %s %s%s%s%s", argv[0],
				"[-c,--config config_file] [-g guid] [-h,--help] [-b,--nobdii] [--srm-timeout value]\n",
				"\t[--connect-timeout value] [--sendreceive-timeout value] [--bdii-timeout value]\n",
				"\t[-D,--defaultsetype se|srmv1|srmv2] [-T,--setype se|srmv1|srmv2]\n",
				"\t[-l lfn] [-v,--verbose] [-V,--vo vo] [--version] surl\n");
		exit (errflg ? 1 : 0);
	}

	c = lcg_rf3 (argv[optind], guid, lfn_arg, defaulttype, setype, nobdii, vo, conf_file,
			insecure, verbose, timeout, actual_guid, NULL, 0);
	if (*actual_guid)
		printf ("guid:%s\n", actual_guid);
	if (c < 0) {
		if (errno)
			perror ("lcg_rf");
		exit (1);
	}
	exit (0);
}
