/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef lint
static char sccsid[] = "@(#)$RCSfile: lcg_aa.c,v $ $Revision: 1.16 $ $Date: 2008/11/10 12:38:16 $ CERN Jean-Philippe Baud";
#endif /* not lint */

/*	lcg_aa - add an alias in RMC for a given GUID */
#include <errno.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <uuid/uuid.h>
#include "gfal_api.h"
#include "lcg_util.h"

lcg_aa (char *lfn, char *guid, char *vo, char *conf_file, int insecure)
{
	return (lcg_aa2 (lfn, guid, 0, vo, NULL, 0));
}

lcg_aax (char *lfn, char *guid, char *vo, char *conf_file, int insecure,
	char *errbuf, int errbufsz)
{
	return (lcg_aa2 (lfn, guid, 0, vo, errbuf, errbufsz));
}

lcg_aa2 (char *lfn, char *guid, int nobdii, char *vo, char *errbuf, int errbufsz)
{
	uuid_t uuid;
	char actual_lfn[GFAL_PATH_MAXLEN];

	if (lfn == NULL || guid == NULL) {
		errno = EFAULT;
		return (-1);
	}
	if (vo && vo[0]) {
		if (strlen (vo) > GFAL_VO_MAXLEN) {
			gfal_errmsg (errbuf, errbufsz, GFAL_ERRLEVEL_ERROR, "%s: VO name too long.", vo);
			errno = ENAMETOOLONG;
			return (-1);
		}
		if (gfal_set_vo (vo) < 0)
			return (-1);
	}

	if (canonical_url (lfn, "lfn", actual_lfn, GFAL_PATH_MAXLEN, errbuf, GFAL_ERRMSG_LEN) < 0)
		return (-1);

	gfal_set_nobdii (nobdii);

	/* check user provided guid */

	if (strncmp (guid, "guid:", 5) == 0)
		guid += 5;	/* strip off guid prefix */
	if (uuid_parse (guid, uuid) < 0) {
		gfal_errmsg (errbuf, errbufsz, GFAL_ERRLEVEL_ERROR, "%s: Invalid GUID.", guid);		
		errno = EINVAL;	/* invalid guid */
		return (-1);
	}

	return (register_alias (guid, actual_lfn + 4, errbuf, errbufsz));
}
