/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifdef lint
static char sccsid[] = "@(#)$RCSfile: lcg_cp.c,v $ $Revision: 1.54 $ $Date: 2009/12/14 15:31:30 $ CERN Jean-Philippe Baud";
#endif /* not lint */

/*	lcg_cp - copy a Grid file to a local destination */
#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <sys/stat.h>
#include <sys/times.h>
#include <unistd.h>
#include <uuid/uuid.h>
#include "lcg_util.h"
#include "gfal_dependencies.h"
#include "globus_dependencies.h"

lcg_cp (char *src_file, char *dest_file, char *vo, int nbstreams,
	char *conf_file, int insecure, int verbose)
{
	return (lcg_cp4 (src_file, dest_file, TYPE_NONE, TYPE_NONE, TYPE_NONE, 0, vo, nbstreams,
				conf_file, insecure, verbose, 0, NULL, NULL, GFAL_CKSM_NONE, NULL, 0));
}


lcg_cpx (char *src_file, char *dest_file, char *vo, int nbstreams,
	char *conf_file, int insecure, int verbose, char *errbuf, int errbufsz)
{
	return (lcg_cp4 (src_file, dest_file, TYPE_NONE, TYPE_NONE, TYPE_NONE, 0, vo, nbstreams,
				conf_file, insecure, verbose, 0, NULL, NULL, GFAL_CKSM_NONE, errbuf, errbufsz));
}


lcg_cpt (char *src_file, char *dest_file, char *vo, int nbstreams,
	char *conf_file, int insecure, int verbose, int timeout)
{
	return (lcg_cp4 (src_file, dest_file, TYPE_NONE, TYPE_NONE, TYPE_NONE, 0, vo, nbstreams,
				conf_file, insecure, verbose, timeout, NULL, NULL, GFAL_CKSM_NONE, NULL, 0));
}


lcg_cpxt (char *src_file, char *dest_file, char *vo, int nbstreams,
	char *conf_file, int insecure, int verbose, int timeout, char *errbuf, int errbufsz)
{
	return (lcg_cp4 (src_file, dest_file, TYPE_NONE, TYPE_NONE, TYPE_NONE, 0, vo, nbstreams,
				conf_file, insecure, verbose, timeout, NULL, NULL, GFAL_CKSM_NONE, errbuf, errbufsz));
}

lcg_cp2 (char *src_file, char *dest_file, char *vo, int nbstreams, char *conf_file,
	 int insecure, int verbose, int timeout, char *src_spacetokendesc, char *dest_spacetokendesc,
	 char *errbuf, int errbufsz)
{
	return (lcg_cp4 (src_file, dest_file, TYPE_NONE, TYPE_NONE, TYPE_NONE, 0, vo, nbstreams,
				conf_file, insecure, verbose, timeout, src_spacetokendesc, dest_spacetokendesc,
				GFAL_CKSM_NONE, errbuf, errbufsz));
}

lcg_cp3 (char *src_file, char *dest_file, enum se_type defaulttype, enum se_type srctype, enum se_type dsttype, int nobdii,
		char *vo, int nbstreams, char *conf_file, int insecure, int verbose, int timeout, char *src_spacetokendesc,
		char *dest_spacetokendesc, char *errbuf, int errbufsz)
{
	return (lcg_cp4 (src_file, dest_file, defaulttype, srctype, dsttype, nobdii, vo, nbstreams, conf_file, insecure,
				verbose, timeout, src_spacetokendesc, dest_spacetokendesc, GFAL_CKSM_NONE, errbuf, errbufsz));
}

lcg_cp4 (char *src_file, char *dest_file, enum se_type defaulttype, enum se_type srctype, enum se_type dsttype, int nobdii,
        char *vo, int nbstreams, char *conf_file, int insecure, int verbose, int timeout, char *src_spacetokendesc,
        char *dest_spacetokendesc, enum gfal_cksm_type cksmtype, char *errbuf, int errbufsz)
{
    return lcg_cp5 (src_file, dest_file, defaulttype, srctype, dsttype, nobdii,
         vo, nbstreams, conf_file, insecure, verbose, timeout, src_spacetokendesc,
         dest_spacetokendesc, cksmtype, NULL, NULL, errbuf, errbufsz);
}

lcg_cp5 (char *src_file, char *dest_file, enum se_type defaulttype,
    enum se_type srctype, enum se_type dsttype, int nobdii, char *vo,
    int nbstreams, char *conf_file, int insecure, int verbose, int timeout,
    char *src_spacetokendesc, char *dest_spacetokendesc,
    enum gfal_cksm_type cksmtype, char** src_protocols, char** dst_protocols,
	char *errbuf, int errbufsz)
{
	off64_t filesize = 0;
	static char *proto4copy[] = {"gsiftp", NULL};
	int n;
	char actual_dest_file[GFAL_PATH_MAXLEN];
	char *dest_surl = NULL;
	char dest_turl[GFAL_PATH_MAXLEN];
	clock_t tm_start = 0;
	struct tms tmp_tms;
	gfal_request src_req = NULL, dst_req = NULL;
	gfal_internal dst_gfal = NULL;
	gfal_filestatus *filestatuses;
	gfal_file gfile_src;
	int bool_issurlok = 0;
	int sav_errno = 0;
	const char *current_surl = NULL;
	enum se_type setype = 0;
	char *srmv2_reqtoken= NULL;
	int srmv1_reqid = 0;
	char *src_cksm = NULL, *dst_cksm = NULL;
	enum gfal_cksm_type tmp_cksmtype = GFAL_CKSM_NONE;
	char errmsg[GFAL_ERRMSG_LEN];

	actual_dest_file[0] = 0;
	dest_turl[0] = 0;

	if (!src_protocols)
	    src_protocols = proto4copy;

	if (!dst_protocols)
	    dst_protocols = proto4copy;

	gfal_set_verbose (verbose);

	if (src_file == NULL || dest_file == NULL) {
		errno = EFAULT;
		return (-1);
	}
	if (nbstreams < 1) {
		gfal_errmsg (errbuf, errbufsz, GFAL_ERRLEVEL_ERROR, "Number of streams must be at least 1");
		errno = EINVAL;
		return (-1);
	}
	if (!strncmp (dest_file, "lfn:", 4) || !strncmp (dest_file, "guid:", 5)) {
		gfal_errmsg (errbuf, errbufsz, GFAL_ERRLEVEL_ERROR, "Destination file cannot be lfn: or guid:.");
		errno = EINVAL;
		return (-1);
	}

	if (vo && vo[0]) {
		if (strlen (vo) > GFAL_VO_MAXLEN) {
			gfal_errmsg (errbuf, errbufsz, GFAL_ERRLEVEL_ERROR, "%s: VO name too long", vo);
			errno = ENAMETOOLONG;
			return (-1);
		}
		if (gfal_set_vo (vo) < 0)
			return (-1);
	}

	if (timeout > 0) {
		// for backward compatibility...
		gfal_set_timeout_connect (timeout);
		gfal_set_timeout_sendreceive (timeout);
		gfal_set_timeout_bdii (timeout);
		gfal_set_timeout_srm (timeout);
	}

	if (canonical_url (dest_file, "file", actual_dest_file, GFAL_PATH_MAXLEN, errbuf, errbufsz) < 0)
		return (-1);

	gfal_set_nobdii (nobdii);

	if ((gfile_src = lcg_gfal_functions.call_gfal_file_new (src_file, "file", 0, errbuf, errbufsz)) == NULL)
		return (-1);
	if (gfile_src->errcode != 0) {
		sav_errno = gfile_src->errcode;
		gfal_file_free (gfile_src);
		errno = sav_errno;
		return (-1);
	}

	gfal_errmsg (NULL, 0, GFAL_ERRLEVEL_WARN, "Using grid catalog type: %s", gfal_file_get_catalog_name (gfile_src));
	gfal_errmsg (NULL, 0, GFAL_ERRLEVEL_WARN, "Using grid catalog : %s", get_catalog_endpoint(errmsg, GFAL_ERRMSG_LEN));
	gfal_errmsg (NULL, 0, GFAL_ERRLEVEL_WARN, "VO name: %s", gfal_get_vo (errmsg, GFAL_ERRMSG_LEN));
	gfal_errmsg (NULL, 0, GFAL_ERRLEVEL_WARN, "Checksum type: %s", gfal_get_checksum_algname (cksmtype));

	if (strncmp (actual_dest_file, "srm:", 4) == 0 || strncmp (actual_dest_file, "sfn:", 4) == 0) {
		dest_surl = (char *) actual_dest_file;
	} else {	/* assume that is a pfn */
		strncpy (dest_turl, actual_dest_file, GFAL_PATH_MAXLEN);
		if (gfal_access (dest_turl, R_OK|W_OK) < 0) {
			if (errno != ENOENT && errno != EPROTONOSUPPORT) {
				sav_errno = errno;
				gfal_file_free (gfile_src);
				gfal_errmsg (errbuf, errbufsz, GFAL_ERRLEVEL_ERROR, "%s: %s",
						dest_turl, strerror (sav_errno));
				errno = sav_errno;
				return (-1);
			} else if (errno == ENOENT) {
				char *p = NULL;

				/* The file doesn't exist, let's check we have write permissions on parent dir. */
				if ((p = strrchr (dest_turl, '/')) != NULL) {
					*p = 0;
					if (gfal_access (dest_turl, R_OK|W_OK) < 0) {
						sav_errno = errno;
						*p = '/';
						gfal_file_free (gfile_src);
						gfal_errmsg (errbuf, errbufsz, GFAL_ERRLEVEL_ERROR, "%s: %s",
								dest_turl, strerror (sav_errno));
						errno = sav_errno;
						return (-1);
					}
					*p = '/';
				} else {
					gfal_file_free (gfile_src);
					gfal_errmsg (errbuf, errbufsz, GFAL_ERRLEVEL_ERROR, "%s: %s",
							dest_turl, strerror (EINVAL));
					errno = EINVAL;
					return (-1);
				}
			}
		}
	}

	/* get source TURL */

	if (gfile_src->turl == NULL) {
		if ((src_req = gfal_request_new ()) == NULL) {
			sav_errno = gfile_src->errcode;
			gfal_file_free (gfile_src);
			errno = sav_errno;
			return (-1);
		}
		src_req->nbfiles = 1;
		src_req->defaultsetype = defaulttype;
		src_req->setype = srctype;
		src_req->no_bdii_check = nobdii;
		src_req->protocols = src_protocols;
		src_req->srmv2_spacetokendesc = src_spacetokendesc;
	}

	while (!bool_issurlok && gfile_src->errcode == 0) {
		bool_issurlok = 1;

		if (gfile_src->nbreplicas > 0) {
			if (gfile_src->gobj == NULL) {
				current_surl = gfal_file_get_replica (gfile_src);
				src_req->surls = (char **) &current_surl;
				gfal_errmsg (NULL, 0, GFAL_ERRLEVEL_WARN, "Trying SURL %s ...", current_surl);

				*errmsg = 0;
				if (!(bool_issurlok = gfal_init (src_req, &gfile_src->gobj, errmsg, GFAL_ERRMSG_LEN) >= 0))
					gfal_file_set_replica_error (gfile_src, errno, errmsg);
			}

			*errmsg = 0;
			if (bool_issurlok && !(bool_issurlok = lcg_gfal_functions.call_gfal_ls (gfile_src->gobj, errmsg, GFAL_ERRMSG_LEN) >= 0))
				gfal_file_set_replica_error (gfile_src, errno, errmsg);

			if (bool_issurlok && (!(bool_issurlok = lcg_gfal_functions.call_gfal_get_results (gfile_src->gobj, &filestatuses) >= 0) ||
						!(bool_issurlok = filestatuses != NULL)))
				gfal_file_set_replica_error (gfile_src, errno, "Internal error");

			if (bool_issurlok && !(bool_issurlok = filestatuses[0].status == 0))
				gfal_file_set_replica_error (gfile_src, filestatuses[0].status, filestatuses[0].explanation);

			if (bool_issurlok) {
				filesize = filestatuses[0].stat.st_size;
				if (cksmtype != GFAL_CKSM_NONE) {
					if ((tmp_cksmtype = gfal_parse_checksum_algname (filestatuses[0].checksumtype)) == cksmtype) {
						/* checksum returned by srmLs is of the same type as the requested one... */
						src_cksm = strdup (filestatuses[0].checksum);
					}
				}
			}

			*errmsg = 0;
			if (bool_issurlok && !(bool_issurlok = lcg_gfal_functions.call_gfal_turlsfromsurls (gfile_src->gobj, errmsg, GFAL_ERRMSG_LEN) >= 0))
				gfal_file_set_replica_error (gfile_src, errno, errmsg);

			if (bool_issurlok && (!(bool_issurlok = lcg_gfal_functions.call_gfal_get_results (gfile_src->gobj, &filestatuses) >= 0) ||
						!(bool_issurlok = filestatuses != NULL)))
				gfal_file_set_replica_error (gfile_src, errno, "Internal error");

			if (bool_issurlok && !(bool_issurlok = filestatuses[0].status == 0))
				gfal_file_set_replica_error (gfile_src, filestatuses[0].status, filestatuses[0].explanation);

			if (bool_issurlok) {
				gfal_errmsg (NULL, 0, GFAL_ERRLEVEL_WARN, "Source SE type: %s", lcg_se_type_string (gfile_src->gobj->setype));

				if (gfal_get_ids_setype (gfile_src->gobj, &setype, &srmv1_reqid, NULL, &srmv2_reqtoken) < 0)
					gfal_errmsg (NULL, 0, GFAL_ERRLEVEL_WARN, "Source SRM Request Token: [ERROR]");
				else if (setype == TYPE_SRM)
					gfal_errmsg (NULL, 0, GFAL_ERRLEVEL_WARN, "Source SRM Request Token: %d", srmv1_reqid);
				else if (setype == TYPE_SRMv2)
					gfal_errmsg (NULL, 0, GFAL_ERRLEVEL_WARN, "Source SRM Request Token: %s", srmv2_reqtoken ? srmv2_reqtoken: "(null)");

				if (srmv2_reqtoken) free (srmv2_reqtoken);

				gfile_src->turl = strdup (filestatuses[0].turl);
				if (!(bool_issurlok = gfile_src->turl != NULL)) {
					sav_errno = errno;
					if (dst_gfal) {
						gfal_abortrequest (dst_gfal, errmsg, GFAL_ERRMSG_LEN);
						gfal_deletesurls (dst_gfal, errmsg, GFAL_ERRMSG_LEN);
						gfal_internal_free (dst_gfal);
					}
					gfal_file_free (gfile_src);
					free (src_req);
					if (src_cksm) free (src_cksm);
					errno = sav_errno;
					return (-1);
				}
			}

			*errmsg = 0;
			if (bool_issurlok && !(bool_issurlok = gfal_set_xfer_running (gfile_src->gobj, errmsg, GFAL_ERRMSG_LEN) >= 0))
				gfal_file_set_replica_error (gfile_src, errno, errmsg);

			if (bool_issurlok && (!(bool_issurlok = lcg_gfal_functions.call_gfal_get_results (gfile_src->gobj, &filestatuses) >= 0) ||
						!(bool_issurlok = filestatuses != NULL)))
				gfal_file_set_replica_error (gfile_src, errno, "Internal error");

			if (bool_issurlok && !(bool_issurlok = filestatuses[0].status == 0))
				gfal_file_set_replica_error (gfile_src, filestatuses[0].status, filestatuses[0].explanation);
		} else if (gfile_src->turl != NULL) {
			struct stat64 statbuf;

			/* Get file size */
			if (!strncmp (gfile_src->turl, "gsiftp:", 7)) {
				*errmsg = 0;
				if (!(bool_issurlok = getfilesizet (gfile_src->turl, &filesize, errmsg,
								GFAL_ERRMSG_LEN, gfal_get_timeout_sendreceive ()) == 0))
					gfal_file_set_turl_error (gfile_src, errno, errmsg);
			} else {
				/* the protocol is different from gridftp:, certainly file: */
				if (!(bool_issurlok = gfal_access (gfile_src->turl, R_OK) == 0))
					gfal_file_set_turl_error (gfile_src, errno, NULL);

				if (bool_issurlok && !(bool_issurlok = gfal_stat64 (gfile_src->turl, &statbuf) == 0))
					gfal_file_set_turl_error (gfile_src, errno, NULL);

				if (bool_issurlok && !(bool_issurlok = !S_ISDIR(statbuf.st_mode)))
					gfal_file_set_turl_error (gfile_src, EISDIR, NULL);

				filesize = statbuf.st_size;

				if (bool_issurlok && cksmtype != GFAL_CKSM_NONE) {
					*errmsg = 0;
					src_cksm = lcg_compute_checksum (gfile_src->turl, cksmtype, errmsg, GFAL_ERRMSG_LEN);
					if (!(bool_issurlok = src_cksm != NULL))
						gfal_file_set_turl_error (gfile_src, errno, errmsg);
				}
			}
		} else bool_issurlok = 0;

		if (bool_issurlok && dest_turl[0] == '\0') {

			/* get destination TURL */

			if (dest_surl) {
				if ((dst_req = gfal_request_new ()) == NULL) {
					sav_errno = errno;
					if (gfile_src->gobj != NULL)
					    lcg_gfal_functions.call_gfal_set_xfer_done (gfile_src->gobj, errbuf, errbufsz);
					gfal_file_free (gfile_src);
					if (src_cksm) free (src_cksm);
					errno = sav_errno;
					return (-1);
				}
				dst_req->nbfiles = 1;
				dst_req->surls = &dest_surl;
				dst_req->defaultsetype = defaulttype;
				dst_req->setype = dsttype;
				dst_req->no_bdii_check = nobdii;
				dst_req->oflag = 1;
				dst_req->filesizes = &filesize;
				dst_req->protocols = dst_protocols;
				dst_req->srmv2_spacetokendesc = dest_spacetokendesc;

				if (gfal_init (dst_req, &dst_gfal, errbuf, errbufsz) < 0) {
					sav_errno = errno;
					if (gfile_src->gobj != NULL)
					    lcg_gfal_functions.call_gfal_set_xfer_done (gfile_src->gobj, errbuf, errbufsz);
					gfal_file_free (gfile_src);
					free (dst_req);
					free (src_req);
					if (src_cksm) free (src_cksm);
					errno = sav_errno;
					return (-1);
				}
				free (dst_req);

				gfal_errmsg (NULL, 0, GFAL_ERRLEVEL_WARN, "Destination SE type: %s", lcg_se_type_string (dst_gfal->setype));

				if (lcg_gfal_functions.call_gfal_turlsfromsurls (dst_gfal, errbuf, errbufsz) < 0) {
					sav_errno = errno;
					free (src_req);
					if (gfile_src->gobj != NULL)
					    lcg_gfal_functions.call_gfal_set_xfer_done (gfile_src->gobj, errbuf, errbufsz);
					gfal_file_free (gfile_src);
					gfal_internal_free (dst_gfal);
					if (src_cksm) free (src_cksm);
					errno = sav_errno;
					return (-1);
				}

				if (gfal_get_ids_setype (dst_gfal, &setype, &srmv1_reqid, NULL, &srmv2_reqtoken) < 0)
					gfal_errmsg (NULL, 0, GFAL_ERRLEVEL_WARN, "Destination SRM Request Token: [ERROR]");
				else if (setype  == TYPE_SRM)
					gfal_errmsg (NULL, 0, GFAL_ERRLEVEL_WARN, "Destination SRM Request Token: %d", srmv1_reqid);
				else if (setype  == TYPE_SRMv2)
					gfal_errmsg (NULL, 0, GFAL_ERRLEVEL_WARN, "Destination SRM Request Token: %s", srmv2_reqtoken ? srmv2_reqtoken: "(null)");

				if (srmv2_reqtoken) free (srmv2_reqtoken);

				if ((n = lcg_gfal_functions.call_gfal_get_results (dst_gfal, &filestatuses)) < 0 || filestatuses == NULL) {
					sav_errno = errno;
					gfal_errmsg (errbuf, errbufsz, GFAL_ERRLEVEL_ERROR, "Internal error");
					free (src_req);
					if (gfile_src->gobj != NULL)
					    lcg_gfal_functions.call_gfal_set_xfer_done (gfile_src->gobj, errbuf, errbufsz);
					gfal_file_free (gfile_src);
					gfal_internal_free (dst_gfal);
					if (src_cksm) free (src_cksm);
					errno = sav_errno;
					return (-1);
				}
				if (filestatuses[0].status != 0) {
					gfal_errmsg (errbuf, errbufsz, GFAL_ERRLEVEL_ERROR, "%s: %s",
							filestatuses[0].surl, strerror (filestatuses[0].status));
					free (src_req);
					if (gfile_src->gobj != NULL)
					    lcg_gfal_functions.call_gfal_set_xfer_done (gfile_src->gobj, errbuf, errbufsz);
					gfal_file_free (gfile_src);
					gfal_internal_free (dst_gfal);
					if (src_cksm) free (src_cksm);
					errno = filestatuses[0].status;
					return (-1);
				}
				strcpy (dest_turl, filestatuses[0].turl);

				if (gfal_set_xfer_running (dst_gfal, errbuf, errbufsz) < 0) {
					sav_errno = errno;
					goto FUNCTION_ERROR_EXIT_WITH_CLEANUP;
				}
				if ((n = lcg_gfal_functions.call_gfal_get_results (dst_gfal, &filestatuses)) < 0 || filestatuses == NULL) {
					sav_errno = errno;
					gfal_errmsg (errbuf, errbufsz, GFAL_ERRLEVEL_ERROR, "Internal error");
					goto FUNCTION_ERROR_EXIT_WITH_CLEANUP;
				}
				if (filestatuses[0].status != 0) {
				    errno = filestatuses[0].status;
					gfal_errmsg (errbuf, errbufsz, GFAL_ERRLEVEL_ERROR, "%s: %s",
							filestatuses[0].surl, strerror (filestatuses[0].status));
					goto FUNCTION_ERROR_EXIT_WITH_CLEANUP;
				}
			}
		}

		if (bool_issurlok && cksmtype != GFAL_CKSM_NONE && src_cksm == NULL) {
			*errmsg = 0;
			if (!(bool_issurlok = gridftp_checksum (gfile_src->turl, cksmtype, &src_cksm, 1,
					errmsg, GFAL_ERRMSG_LEN, gfal_get_timeout_sendreceive ()) == 0 && src_cksm)) {
				sav_errno = errno;
				gfal_file_set_turl_error (gfile_src, sav_errno, errmsg);
				if (gfile_src->gobj != NULL)
				    lcg_gfal_functions.call_gfal_set_xfer_done (gfile_src->gobj, errmsg, GFAL_ERRMSG_LEN);
			}
		}

		if (bool_issurlok) {
			char tmpbuf[21];
			gfal_errmsg (NULL, 0, GFAL_ERRLEVEL_WARN, "Source URL: %s", gfile_src->file);
			sprintf (tmpbuf, GFAL_LONG64_FORMAT, filesize);
			gfal_errmsg (NULL, 0, GFAL_ERRLEVEL_WARN, "File size: %s", tmpbuf);
			gfal_errmsg (NULL, 0, GFAL_ERRLEVEL_WARN, "Source URL for copy: %s", gfile_src->turl);
			gfal_errmsg (NULL, 0, GFAL_ERRLEVEL_WARN, "Destination URL: %s", dest_turl);
			gfal_errmsg (NULL, 0, GFAL_ERRLEVEL_WARN, "# streams: %d", nbstreams);

			if (verbose)
				tm_start = times (&tmp_tms);

			/* copy file */

			*errmsg = 0;
			if (!(bool_issurlok = lcg_globus_functions.call_globus_copyfilex (gfile_src->turl, dest_turl, nbstreams, errmsg,
							GFAL_ERRMSG_LEN, gfal_get_timeout_sendreceive (), verbose) == 0)) {
				sav_errno = errno;
				if (sav_errno == ECANCELED) {
					gfal_errmsg (errbuf, errbufsz, GFAL_ERRLEVEL_ERROR, "Copy Cancelled...");

					if (gfile_src->gobj != NULL)
					    lcg_gfal_functions.call_gfal_set_xfer_done (gfile_src->gobj, errmsg, GFAL_ERRMSG_LEN);

					if (strncmp (actual_dest_file, "srm", 3) == 0) {
						gfal_abortrequest (dst_gfal, errmsg, GFAL_ERRMSG_LEN);
						gfal_deletesurls (dst_gfal, errmsg, GFAL_ERRMSG_LEN);
					}

					free (src_req);
					gfal_file_free (gfile_src);
					gfal_internal_free (dst_gfal);
					if (src_cksm) free (src_cksm);
					errno = sav_errno;
					return (-1);
				}

				gfal_file_set_turl_error (gfile_src, sav_errno, errmsg);
				if (gfile_src->gobj != NULL)
				    lcg_gfal_functions.call_gfal_set_xfer_done (gfile_src->gobj, errmsg, GFAL_ERRMSG_LEN);
			} else
				gfal_errmsg (NULL, 0, GFAL_ERRLEVEL_WARN, "\nTransfer took %d ms",
						(int) ((times (&tmp_tms) - tm_start) / (sysconf (_SC_CLK_TCK) / 1000.)));
		}

		if (!bool_issurlok) {
			if (lcg_gfal_functions.call_gfal_file_next_replica (gfile_src) < 0) {
			    gfal_errmsg (errbuf, errbufsz, GFAL_ERRLEVEL_ERROR, gfile_src->errmsg);
	            errno = gfile_src->errcode;
                goto FUNCTION_ERROR_EXIT_WITH_CLEANUP;
			}
		}
	}
	free (src_req);

	if (gfile_src->errcode) {
		sav_errno = gfile_src->errcode;

		if (gfile_src->errmsg && gfile_src->errmsg[0])
			gfal_errmsg (errbuf, errbufsz, GFAL_ERRLEVEL_ERROR, "%s", gfile_src->errmsg);
		else
			gfal_errmsg (errbuf, errbufsz, GFAL_ERRLEVEL_ERROR, "%s: %s", gfile_src->file,
					strerror (sav_errno));

		if (strncmp (actual_dest_file, "srm", 3) == 0) {
			gfal_abortrequest (dst_gfal, errmsg, GFAL_ERRMSG_LEN);
			gfal_deletesurls (dst_gfal, errmsg, GFAL_ERRMSG_LEN);
		}
		gfal_file_free (gfile_src);
		gfal_internal_free (dst_gfal);
		if (src_cksm) free (src_cksm);
		errno = sav_errno;
		return (-1);
	}

	if (gfile_src->gobj != NULL)
	    lcg_gfal_functions.call_gfal_set_xfer_done (gfile_src->gobj, errbuf, errbufsz);

	gfal_file_free (gfile_src);

	if (strncmp (actual_dest_file, "srm", 3) == 0) {
		if (lcg_gfal_functions.call_gfal_set_xfer_done (dst_gfal, errbuf, errbufsz) < 0) {
			sav_errno = errno;
			gfal_internal_free (dst_gfal);
			if (src_cksm) free (src_cksm);
			errno = sav_errno;
			return (-1);
		}
		if ((n = lcg_gfal_functions.call_gfal_get_results (dst_gfal, &filestatuses)) < 0 || filestatuses == NULL) {
			sav_errno = errno;
			gfal_errmsg (errbuf, errbufsz, GFAL_ERRLEVEL_ERROR, "Internal error");
			gfal_internal_free (dst_gfal);
			if (src_cksm) free (src_cksm);
			errno = sav_errno;
			return (-1);
		}
		if (filestatuses[0].status != 0) {
			gfal_errmsg (errbuf, errbufsz, GFAL_ERRLEVEL_ERROR, "%s: %s",
					filestatuses[0].surl, strerror (filestatuses[0].status));
			gfal_internal_free (dst_gfal);
			if (src_cksm) free (src_cksm);
			errno = filestatuses[0].status;
			return (-1);
		}
	}

	sav_errno = 0;

	if (cksmtype != GFAL_CKSM_NONE) {
		lcg_checksum *checksums = NULL;

		if (dst_gfal) {
			lcg_get_checksum_surls (dst_gfal, 0, cksmtype, &checksums, errbuf, errbufsz);

			if (checksums == NULL) {
				sav_errno = errno ? errno : EFAULT;
			}
			else if (checksums[0].status != 0) {
				sav_errno = checksums[0].status;
				if (checksums[0].explanation)
					gfal_errmsg (errbuf, errbufsz, GFAL_ERRLEVEL_ERROR, "%s", checksums[0].explanation);
				else
					gfal_errmsg (errbuf, errbufsz, GFAL_ERRLEVEL_ERROR, "[LCG-UTIL][lcg_get_checksum_surls][] %s: %s",
							checksums[0].file, strerror (checksums[0].status));
			}
			else if (checksums[0].checksum == NULL) {
				sav_errno = EFAULT;
			}
			else
				dst_cksm = checksums[0].checksum;

			if (checksums) {
				if (checksums[0].file) free (checksums[0].file);
				if (checksums[0].explanation) free (checksums[0].explanation);
				free (checksums);
			}
		}
		else {
			dst_cksm = lcg_compute_checksum (actual_dest_file, cksmtype, errbuf, errbufsz);
			if (dst_cksm == NULL)
				sav_errno = errno ? errno : EFAULT;
		}

		if (!sav_errno)
		{
            lcgutil_str_tolower(src_cksm);
            lcgutil_str_tolower(dst_cksm);

			if (strcmp (src_cksm, dst_cksm)) {
				/* checksums are different!!! */
				sav_errno = EAGAIN;
				gfal_unlink (actual_dest_file);
				gfal_errmsg (errbuf, errbufsz, GFAL_ERRLEVEL_ERROR,
						"[LCG-UTIL][lcg_cp4][] Destination may be corrupted: Source checksum (%s) != Destination checksum (%s)",
						src_cksm, dst_cksm);
				gfal_errmsg (errbuf, errbufsz, GFAL_ERRLEVEL_ERROR,
						"[LCG-UTIL][lcg_cp4][] %s has been DELETED, please try again (temporary network problem?)", dest_file);
			}
			else
				gfal_errmsg (errbuf, errbufsz, GFAL_ERRLEVEL_WARN,
						"[INFO] Source and destination checksums are matching: %s", src_cksm);
		}

		free (src_cksm);
		free (dst_cksm);
	}

	gfal_internal_free (dst_gfal);
	errno = sav_errno;
	return (sav_errno ? -1 : 0);

FUNCTION_ERROR_EXIT_WITH_CLEANUP:
    sav_errno = errno;
    free (src_req);
    if (gfile_src->gobj != NULL)
        lcg_gfal_functions.call_gfal_set_xfer_done (gfile_src->gobj, errbuf, errbufsz);
    gfal_abortrequest (dst_gfal, errmsg, GFAL_ERRMSG_LEN);
    gfal_deletesurls (dst_gfal, errmsg, GFAL_ERRMSG_LEN);
    gfal_file_free (gfile_src);
    gfal_internal_free (dst_gfal);
    if (src_cksm)
        free (src_cksm);
    errno = sav_errno;
    return (-1);
}
