/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef lint
static char sccsid[] = "@(#)$RCSfile: lcg_cr.c,v $ $Revision: 1.75 $ $Date: 2010/01/21 08:44:26 $ CERN Jean-Philippe Baud";
#endif /* not lint */

/*	lcg_cr - copy a file to a Storage Element and registers it in the LRC
 *	with a specified GUID
 */
#include <errno.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <sys/stat.h>
#include <sys/times.h>
#include <stdio.h>
#include <unistd.h>
#include <uuid/uuid.h>
#include "lcg_util.h"
#include "gfal_dependencies.h"
#include "globus_dependencies.h"

lcg_cr (char *src_file, char *dest_file, char *guid, char *lfn, char *vo,
		char *relative_path, int nbstreams, char *conf_file, int insecure,
		int verbose, char *actual_guid)
{
	return (lcg_cr4 (src_file, dest_file, guid, lfn, TYPE_NONE, TYPE_NONE, 0, vo, relative_path,
				nbstreams, conf_file, insecure, verbose, 0, NULL, actual_guid, GFAL_CKSM_NONE, NULL, 0));
}

lcg_crt (char *src_file, char *dest_file, char *guid, char *lfn, char *vo,
		char *relative_path, int nbstreams, char *conf_file, int insecure,
		int verbose, int timeout, char *actual_guid)
{
	return (lcg_cr4 (src_file, dest_file, guid, lfn, TYPE_NONE, TYPE_NONE, 0, vo, relative_path,
				nbstreams, conf_file, insecure, verbose, timeout, NULL, actual_guid,
				GFAL_CKSM_NONE, NULL, 0));
}


lcg_crx (char *src_file, char *dest_file, char *guid, char *lfn, char *vo,
		char *relative_path, int nbstreams, char *conf_file, int insecure,
		int verbose,  char *actual_guid, char *errbuf, int errbufsz)
{
	return (lcg_cr4 (src_file, dest_file, guid, lfn, TYPE_NONE, TYPE_NONE, 0, vo, relative_path,
				nbstreams, conf_file, insecure, verbose, 0, NULL, actual_guid,
				GFAL_CKSM_NONE, errbuf, errbufsz));
}

lcg_crxt (char *src_file, char *dest_file, char *guid, char *lfn, char *vo,
		char *relative_path, int nbstreams, char *conf_file, int insecure,
		int verbose, int timeout, char *actual_guid, char *errbuf, int errbufsz)
{
	return (lcg_cr4 (src_file, dest_file, guid, lfn, TYPE_NONE, TYPE_NONE, 0, vo, relative_path,
				nbstreams, conf_file, insecure, verbose, timeout, NULL, actual_guid,
				GFAL_CKSM_NONE, errbuf, errbufsz));
}

lcg_cr2 (char *src_file, char *dest_file, char *guid, char *lfn, char *vo,
		char *relative_path, int nbstreams, char *conf_file, int insecure,
		int verbose, int timeout, char *spacetokendesc, char *actual_guid, char *errbuf, int errbufsz)
{
	return (lcg_cr4 (src_file, dest_file, guid, lfn, TYPE_NONE, TYPE_NONE, 0, vo, relative_path,
				nbstreams, conf_file, insecure, verbose, timeout, spacetokendesc, actual_guid,
				GFAL_CKSM_NONE, errbuf, errbufsz));
}

lcg_cr3 (char *src_file, char *dest_file, char *guid, char *lfn, enum se_type defaulttype, enum se_type dst_setype,
		int nobdii, char *vo, char *relative_path, int nbstreams, char *conf_file, int insecure,
		int verbose, int timeout, char *spacetokendesc, char *actual_guid, char *errbuf, int errbufsz)
{
	return (lcg_cr4 (src_file, dest_file, guid, lfn, defaulttype, dst_setype, nobdii, vo, relative_path,
				nbstreams, conf_file, insecure, verbose, timeout, spacetokendesc, actual_guid, GFAL_CKSM_NONE,
				errbuf, errbufsz));
}

int lcg_cr4 (char *src_file, char *dest_file, char *guid, char *lfn,
    enum se_type defaulttype, enum se_type dst_setype, int nobdii, char *vo,
    char *relative_path, int nbstreams, char *conf_file, int insecure,
    int verbose, int timeout, char *spacetokendesc, char *actual_guid,
    enum gfal_cksm_type cksmtype, char *errbuf, int errbufsz)
{
    return (lcg_cr5 (src_file, dest_file, guid, lfn, defaulttype, dst_setype,
        nobdii, vo, relative_path, nbstreams, conf_file, insecure, verbose,
        timeout, spacetokendesc, actual_guid, GFAL_CKSM_NONE, NULL,
        errbuf, errbufsz));
}
int lcg_cr5 (char *src_file, char *dest_file, char *guid, char *lfn,
    enum se_type defaulttype, enum se_type dst_setype, int nobdii, char *vo,
    char *relative_path, int nbstreams, char *conf_file, int insecure,
    int verbose, int timeout, char *spacetokendesc, char *actual_guid,
    enum gfal_cksm_type cksmtype, char** dst_protocols,
    char *errbuf, int errbufsz)
{
	return (lcg_cr6 (src_file, dest_file, guid, lfn, defaulttype, dst_setype,
	        nobdii, vo, relative_path, nbstreams, conf_file, insecure, verbose,
	        timeout, spacetokendesc, actual_guid, GFAL_CKSM_NONE, dst_protocols, 1,
	        errbuf, errbufsz));
}

/* Check is the LFN already exists.
   Origin: Ticket 161:  lcg-cr gives vague error for existing LFN
   (https://svnweb.cern.ch/trac/lcgutil/ticket/161
*/
static int is_lfn_exists_ (const char* lfn)
{
    int res = 1;
    int statres = 0;
    struct stat64 buf;

    statres = lfc_lstatl_silent (lfn, &buf);

    if (statres != -1)
    {
        res = 0;
    }

	return res;
}

int lcg_cr6 (char *src_file, char *dest_file, char *guid, char *lfn,
    enum se_type defaulttype, enum se_type dst_setype, int nobdii, char *vo,
    char *relative_path, int nbstreams, char *conf_file, int insecure,
    int verbose, int timeout, char *spacetokendesc, char *actual_guid,
    enum gfal_cksm_type cksmtype, char** dst_protocols,int nofilesizecheck,
    char *errbuf, int errbufsz)
{
	char actual_src_file[GFAL_PATH_MAXLEN];
	char dest_turl[GFAL_PATH_MAXLEN];
	char *generated_guid = NULL;
	off64_t filesize_src,filesize_dest;
	int n;
	static char *proto4copy[] = {"gsiftp", NULL};
	clock_t tm_start = 0;
	struct tms tmp_tms;
	char *cat_type = NULL;
	char cattype[8];
	char actual_lfn[GFAL_PATH_MAXLEN];
	gfal_request req = NULL;
	gfal_internal gfal = NULL;
	gfal_filestatus *filestatuses;
	char errmsg[GFAL_ERRMSG_LEN];
	int sav_errno = 0;
	struct stat64 statbuf;
	char *src_cksm = NULL, *dst_cksm = NULL;
	int res = 0;

	gfal_set_verbose (verbose);

	if (!dst_protocols)
	     dst_protocols = proto4copy;

	if (actual_guid)
		*actual_guid = '\0';
	if (src_file == NULL) {
		errno = EINVAL;
		return (-1);
	}

	if (nbstreams < 1) {
		gfal_errmsg (errbuf, errbufsz, GFAL_ERRLEVEL_ERROR, "Number of streams < 1.");
		errno = EINVAL;
		return (-1);
	}
	if (vo && vo[0]) {
		if (strlen (vo) > GFAL_VO_MAXLEN) {
			gfal_errmsg (errbuf, errbufsz, GFAL_ERRLEVEL_ERROR, "VO name too long.");
			errno = ENAMETOOLONG;
			return (-1);
		}
		if (gfal_set_vo (vo) < 0)
			return (-1);
	}

	if (timeout > 0) {
		// for backward compatibility...
		gfal_set_timeout_connect (timeout);
		gfal_set_timeout_sendreceive (timeout);
		gfal_set_timeout_bdii (timeout);
		gfal_set_timeout_srm (timeout);
	}

	if (canonical_url (src_file, "file", actual_src_file, GFAL_PATH_MAXLEN, errbuf, errbufsz) < 0)
		return (-1);

	if (strncmp (actual_src_file, "file:",5) && strncmp (src_file, "gsiftp:",7)) {
		gfal_errmsg (errbuf, errbufsz, GFAL_ERRLEVEL_ERROR, "Source file must start by 'file:' or 'gsiftp:'.");
		errno = EINVAL;
		return (-1);
	}

	gfal_set_nobdii (nobdii);

	/* check for dest file, if not there */
	if(dest_file == NULL) {
		errno = 0;
		if ((dest_file = get_default_se (errbuf, errbufsz)) == NULL) {
			if (errno == 0)
				gfal_errmsg (errbuf, errbufsz, GFAL_ERRLEVEL_ERROR, "No destination SE specified, and no default SE specified in environment");
			return (-1);
		}
	}

	if (get_cat_type (&cat_type) < 0 ) {
		return (-1);
	}
	if (strcmp (cat_type, "edg") && strcmp (cat_type, "lfc")) {
	       if(cat_type)
			free (cat_type);
		gfal_errmsg (errbuf, errbufsz, GFAL_ERRLEVEL_ERROR, "The catalog type is neither 'edg' nor 'lfc'.");
		errno = EINVAL;
		return (-1);
	}
	strcpy (cattype, cat_type);
	if(cat_type)
		free (cat_type);

	gfal_errmsg (NULL, 0, GFAL_ERRLEVEL_WARN, "Using grid catalog type: %s", cattype);
	gfal_errmsg (NULL, 0, GFAL_ERRLEVEL_WARN, "Using grid catalog : %s", get_catalog_endpoint (errmsg, GFAL_ERRMSG_LEN));
	gfal_errmsg (NULL, 0, GFAL_ERRLEVEL_WARN, "Checksum type: %s", gfal_get_checksum_algname (cksmtype));

	actual_lfn[0] = 0;
	if (lfn && lfn[0]) {
		if (canonical_url (lfn, "lfn", actual_lfn, GFAL_PATH_MAXLEN, errbuf, errbufsz) < 0)
			return (-1);

		if(strncmp (actual_lfn, "lfn:", 4) == 0) {
			memmove (actual_lfn, actual_lfn + 4, strlen (actual_lfn + 4) + 1);
		} else {
			gfal_errmsg (errbuf, errbufsz, GFAL_ERRLEVEL_ERROR, "%s: Must be a lfn", actual_lfn);
			errno = EINVAL;
			return (-1);
		}
	}

    /* Check is the LFN already exists.
       Origin: Ticket 161:  lcg-cr gives vague error for existing LFN
       (https://svnweb.cern.ch/trac/lcgutil/ticket/161
    */
    res = is_lfn_exists_ (actual_lfn);

    if (!res)
    {
        gfal_errmsg (errbuf, errbufsz, GFAL_ERRLEVEL_ERROR,
            "Cannot execute command, strong suspicion: LFN already exists, you should check it.");

        return (-1);
    }

	/* get file size */
	if (!strncmp (actual_src_file, "gsiftp:", 7)) {
		if (lcg_globus_functions.call_globus_getfilesizet (actual_src_file, &filesize_src, errbuf, errbufsz, gfal_get_timeout_sendreceive ()) < 0)
			return (-1);
	} else {
		if (gfal_access (actual_src_file, R_OK) < 0) {
			gfal_errmsg (errbuf, errbufsz, GFAL_ERRLEVEL_ERROR, "%s: %s", src_file, strerror (errno));
			return (-1);
		}
		if (gfal_stat64 (actual_src_file, &statbuf) < 0)
			return (-1);
		if (S_ISDIR(statbuf.st_mode)) {
			gfal_errmsg (errbuf, errbufsz, GFAL_ERRLEVEL_ERROR, "%s: Is a directory", src_file);
			errno = EISDIR;
			return (-1);
		}
		filesize_src = statbuf.st_size;
	}

	if (cksmtype != GFAL_CKSM_NONE) {
		src_cksm = lcg_compute_checksum (actual_src_file, cksmtype, errbuf, errbufsz);
		if (src_cksm == NULL) {
			errno = errno ? errno : EFAULT;
			return (-1);
		}
	}

	if ((req = gfal_request_new ()) == NULL)
		return (-1);
	req->relative_path = relative_path;
	req->nbfiles = 1;
	req->oflag = 1;
	req->filesizes = &filesize_src;
	req->defaultsetype = defaulttype;
	req->setype = dst_setype;
	req->no_bdii_check = nobdii;
	req->protocols = dst_protocols;
	req->srmv2_spacetokendesc = spacetokendesc;

	if (strstr (dest_file, ":/") == NULL) {
		/* destination is just a hostname, one must generate a SURL */
		req->endpoint = dest_file;
		req->generatesurls = 1;
	} else {
		if (strncmp (dest_file, "srm:/", 5) &&
				strncmp (dest_file, "sfn:/", 5)) {
			gfal_errmsg (errbuf, errbufsz, GFAL_ERRLEVEL_ERROR, "Destination URL starts with neither 'srm:/' nor 'sfn:/'.");
			if (src_cksm) free (src_cksm);
			errno = EINVAL;
			return (-1);
		}
		req->surls = (char **) &dest_file;
	}

	if (lcg_gfal_functions.call_gfal_init (req, &gfal, errbuf, errbufsz) < 0) {
		sav_errno = errno;
		if(req)
			free (req);
		if (src_cksm)
			free (src_cksm);
		errno = sav_errno;
		return (-1);
	}
	if(req)
		free (req);

	gfal_errmsg (NULL, 0, GFAL_ERRLEVEL_WARN, "SE type: %s", lcg_se_type_string (gfal->setype));
	gfal_errmsg (NULL, 0, GFAL_ERRLEVEL_WARN, "Destination SURL : %s", gfal->surls[0]);

	/* get destination TURL */

	if (lcg_gfal_functions.call_gfal_turlsfromsurls (gfal, errbuf, errbufsz) < 0) {
		sav_errno = errno;
		if(gfal){
		  gfal_internal_free (gfal);
		  gfal=NULL;
		  }
		if (src_cksm) free (src_cksm);
		errno = sav_errno;
		return (-1);
	}

	{
		char *srmv2_reqtoken = NULL;
		int srmv1_reqid = 0;
		enum se_type setype = 0;

		if (gfal_get_ids_setype (gfal, &setype, &srmv1_reqid, NULL, &srmv2_reqtoken) < 0)
			gfal_errmsg (NULL, 0, GFAL_ERRLEVEL_WARN, "Source SRM Request Token: [ERROR]");
		else if (setype == TYPE_SRM)
			gfal_errmsg (NULL, 0, GFAL_ERRLEVEL_WARN, "Source SRM Request Token: %d", srmv1_reqid);
		else if (setype == TYPE_SRMv2)
			gfal_errmsg (NULL, 0, GFAL_ERRLEVEL_WARN, "Source SRM Request Token: %s", srmv2_reqtoken ? srmv2_reqtoken: "(null)");

		if (srmv2_reqtoken) free (srmv2_reqtoken);
	}

	if ((n = lcg_gfal_functions.call_gfal_get_results (gfal, &filestatuses)) < 0 || filestatuses == NULL) {
		sav_errno = errno;
		if(gfal){
			gfal_internal_free (gfal);
			gfal=NULL;
			}
		gfal_errmsg (errbuf, errbufsz, GFAL_ERRLEVEL_ERROR, "Internal error");
		if (src_cksm) free (src_cksm);
		errno = sav_errno;
		return (-1);
	}
	if (filestatuses[0].status != 0) {
		gfal_errmsg (errbuf, errbufsz, GFAL_ERRLEVEL_ERROR, "%s: %s",
				filestatuses[0].surl, strerror (filestatuses[0].status));
		if(gfal){
			gfal_internal_free (gfal);
			gfal=NULL;
			}
		if (src_cksm) free (src_cksm);
		errno = filestatuses[0].status;
		return (-1);
	}
	strcpy (dest_turl, filestatuses[0].turl);

	{
		char tmpbuf[21];
		gfal_errmsg (NULL, 0, GFAL_ERRLEVEL_WARN, "Source URL: %s", actual_src_file);
		sprintf (tmpbuf, GFAL_LONG64_FORMAT, filesize_src);
		gfal_errmsg (NULL, 0, GFAL_ERRLEVEL_WARN, "File size: %s", tmpbuf);
		gfal_errmsg (NULL, 0, GFAL_ERRLEVEL_WARN, "VO name: %s", gfal_get_vo (errmsg, GFAL_ERRMSG_LEN));
		gfal_errmsg (NULL, 0, GFAL_ERRLEVEL_WARN, "Destination specified: %s", dest_file);
		gfal_errmsg (NULL, 0, GFAL_ERRLEVEL_WARN, "Destination URL for copy: %s", dest_turl);
		gfal_errmsg (NULL, 0, GFAL_ERRLEVEL_WARN, "# streams: %d", nbstreams);
	}

	/* copy file */

	if (gfal_set_xfer_running (gfal, errbuf, errbufsz) < 0) {
		sav_errno = errno;
		if(gfal){
			gfal_internal_free (gfal);
			gfal=NULL;
			}
		if (src_cksm) free (src_cksm);
		errno = sav_errno;
		return (-1);
	}
	if ((n = lcg_gfal_functions.call_gfal_get_results (gfal, &filestatuses)) < 0 || filestatuses == NULL) {
		sav_errno = errno;
		if(gfal){
			gfal_internal_free (gfal);
			gfal=NULL;
			}
		gfal_errmsg (errbuf, errbufsz, GFAL_ERRLEVEL_ERROR, "Internal error");
		if (src_cksm) free (src_cksm);
		errno = sav_errno;
		return (-1);
	}
	if (filestatuses[0].status != 0) {
		gfal_errmsg (errbuf, errbufsz, GFAL_ERRLEVEL_ERROR, "%s: %s",
				filestatuses[0].surl, strerror (filestatuses[0].status));
		if(gfal){
			gfal_internal_free (gfal);
			gfal=NULL;
			}
		if (src_cksm) free (src_cksm);
		errno = filestatuses[0].status;
		return (-1);
	}

	if (guid == NULL && (guid = generated_guid = gfal_generate_guid (errbuf, errbufsz)) == NULL) {
		sav_errno = errno;
		if(gfal){
			gfal_internal_free (gfal);
			gfal=NULL;
			}
		if (src_cksm) free (src_cksm);
		errno = sav_errno;
		return (-1);
	}

	if (verbose)
		tm_start = times (&tmp_tms);

	if (lcg_globus_functions.call_globus_copyfilex (actual_src_file, dest_turl, nbstreams, errbuf, errbufsz, gfal_get_timeout_sendreceive (), verbose) < 0) {
		sav_errno = errno;
	  	if (sav_errno == ECANCELED)
			gfal_errmsg (errbuf, errbufsz, GFAL_ERRLEVEL_ERROR, "Copy Cancelled...");
		else
			gfal_errmsg (errbuf, errbufsz, GFAL_ERRLEVEL_ERROR, "%s: %s", dest_turl, strerror (sav_errno));

		gfal_abortrequest (gfal, errmsg, GFAL_ERRMSG_LEN);
		gfal_deletesurls (gfal, errmsg, GFAL_ERRMSG_LEN);
		if(gfal){
			gfal_internal_free (gfal);
			gfal=NULL;
			}
		if (generated_guid) free (generated_guid);
		if (src_cksm) free (src_cksm);
		errno = sav_errno;
		return (-1);
	}
	gfal_errmsg (NULL, 0, GFAL_ERRLEVEL_WARN, "\nTransfer took %d ms",
			(int) ((times (&tmp_tms) - tm_start) / (sysconf (_SC_CLK_TCK) / 1000.)));

	if (actual_guid)
		strcpy (actual_guid, guid);

	if (lcg_gfal_functions.call_gfal_set_xfer_done (gfal, errbuf, errbufsz) < 0) {
		sav_errno = errno ? errno : ECOMM;
		if(gfal){
			gfal_internal_free (gfal);
			gfal=NULL;
			}
	} else if ((n = lcg_gfal_functions.call_gfal_get_results (gfal, &filestatuses)) < 0 || filestatuses == NULL) {
		sav_errno = errno ? errno : EFAULT;
		if(gfal){
			gfal_internal_free (gfal);
			gfal=NULL;
			}
		gfal_errmsg (errbuf, errbufsz, GFAL_ERRLEVEL_ERROR, "Internal error");
	} else if (filestatuses[0].status != 0) {
		gfal_errmsg (errbuf, errbufsz, GFAL_ERRLEVEL_ERROR, "%s: %s",
				filestatuses[0].surl, strerror (filestatuses[0].status));

		sav_errno = filestatuses[0].status;
		gfal_deletesurls (gfal, errbuf, errbufsz);
		if(gfal){
			gfal_internal_free (gfal);
			gfal=NULL;
			}
	}

	if (cksmtype != GFAL_CKSM_NONE) {
		lcg_checksum *checksums = NULL;

		lcg_get_checksum_surls (gfal, 0, cksmtype, &checksums, errbuf, errbufsz);

		if (checksums == NULL)
			sav_errno = errno ? errno : EFAULT;
		else if (checksums[0].status != 0) {
			sav_errno = checksums[0].status;
			if (checksums[0].explanation)
				gfal_errmsg (errbuf, errbufsz, GFAL_ERRLEVEL_ERROR, "%s", checksums[0].explanation);
			else
				gfal_errmsg (errbuf, errbufsz, GFAL_ERRLEVEL_ERROR, "[LCG-UTIL][lcg_get_checksum_surls][] %s: %s",
						checksums[0].file, strerror (checksums[0].status));
		}
		else if (checksums[0].checksum == NULL)
			sav_errno = EFAULT;
		else
			dst_cksm = checksums[0].checksum;

		if (checksums) {
			if (checksums[0].file) free (checksums[0].file);
			if (checksums[0].explanation) free (checksums[0].explanation);
			free (checksums);
		}

		if (!sav_errno)
		{
            lcgutil_str_tolower(src_cksm);
            lcgutil_str_tolower(dst_cksm);

			if (strcmp (src_cksm, dst_cksm)) {
				/* checksums are different!!! */
				sav_errno = EAGAIN;

                if (actual_guid) {
    				actual_guid[0] = '\0';
                }

				gfal_deletesurls (gfal, errbuf, errbufsz);
				gfal_errmsg (errbuf, errbufsz, GFAL_ERRLEVEL_ERROR,
						"[LCG-UTIL][lcg_cr4][] Destination may be corrupted: Source checksum (%s) != Destination checksum (%s)",
						src_cksm, dst_cksm);
				gfal_errmsg (errbuf, errbufsz, GFAL_ERRLEVEL_ERROR,
						"[LCG-UTIL][lcg_cp4][] %s has been DELETED, please try again (temporary network problem?)", gfal->surls[0]);
			}
			else
				gfal_errmsg (errbuf, errbufsz, GFAL_ERRLEVEL_WARN,
						"[INFO] Source and destination checksums are matching: %s", src_cksm);
		}
	if(src_cksm)
		free (src_cksm);
	if(dst_cksm)
		free (dst_cksm);
	}

	if (!sav_errno) {

		if (lcg_gfal_functions.call_gfal_ls (gfal, errbuf, errbufsz) < 0) {
			if (generated_guid) free (generated_guid);
			if(gfal){
				gfal_internal_free (gfal);
				gfal=NULL;
				}
			return (-1);
		}
		if (lcg_gfal_functions.call_gfal_get_results (gfal, &filestatuses) < 0 || filestatuses == NULL) {
			if (generated_guid) free (generated_guid);
			if(gfal){
				gfal_internal_free (gfal);
				gfal=NULL;
				}
			gfal_errmsg (errbuf, errbufsz, GFAL_ERRLEVEL_ERROR, "Internal error");
			return (-1);
		}
		if (filestatuses[0].status != 0) {
			gfal_errmsg (errbuf, errbufsz, GFAL_ERRLEVEL_ERROR, "%s: %s",
					filestatuses[0].surl, strerror (filestatuses[0].status));
			if (generated_guid) free (generated_guid);
			errno = filestatuses[0].status;
			if(gfal){
				gfal_internal_free (gfal);
				gfal=NULL;
				}
			return (-1);
		}
		filesize_dest = filestatuses[0].stat.st_size;
		{
			char tmpbuf[21];
			gfal_errmsg (NULL, 0, GFAL_ERRLEVEL_WARN, "Using grid catalog type: %s", cattype);
			gfal_errmsg (NULL, 0, GFAL_ERRLEVEL_WARN, "Using grid catalog : %s", get_catalog_endpoint(errbuf, errbufsz));
			gfal_errmsg (NULL, 0, GFAL_ERRLEVEL_WARN, "Site URL to be registered: %s",  gfal->surls[0]);
			sprintf (tmpbuf, GFAL_LONG64_FORMAT, filesize_dest);
			gfal_errmsg (NULL, 0, GFAL_ERRLEVEL_WARN, "File size: %s", tmpbuf);
		}
		if (!nofilesizecheck)
		{
			if (filesize_dest != filesize_src)
			{
                if (actual_guid) {
    				actual_guid[0] = '\0';
                }
				gfal_deletesurls (gfal, errbuf, errbufsz);
				gfal_errmsg (errbuf, errbufsz, GFAL_ERRLEVEL_ERROR,
														"%s: Registration failed, file size mismatch.", gfal->surls[0]);
				sav_errno = errno != 0 ? errno : EINVAL;
			}
		}
		if (!sav_errno)
		{
			// Put the LFN into the catalog
			if (lcg_gfal_functions.call_gfal_register_file (actual_lfn, guid, gfal->surls[0], 0, filesize_dest, 1, errbuf, errbufsz) < 0 ) {
				if (actual_guid) {
                    actual_guid[0] = '\0';
                }

				lcg_gfal_functions.call_gfal_deletesurls(gfal, errbuf, errbufsz);
				sav_errno = errno != 0 ? errno : ECOMM;
				switch (errno)
				{
					case EEXIST:
						gfal_errmsg (errbuf, errbufsz, GFAL_ERRLEVEL_ERROR,
											"%s: Registration failed, file already exists.", gfal->surls[0]);
						break;
					default:
						gfal_errmsg (errbuf, errbufsz, GFAL_ERRLEVEL_ERROR,
											"%s: Registration failed, please register it by hand, when the problem will be solved", gfal->surls[0]);
						break;
				}
			}
		}
	}

	if (generated_guid) free (generated_guid);
	if(gfal){
		gfal_internal_free (gfal);
		gfal=NULL;
		}
	errno = sav_errno;
	return (sav_errno ? -1 : 0);
}
