/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef lint
static char sccsid[] = "@(#)$RCSfile: lcg_gt.c,v $ $Revision: 1.27 $ $Date: 2008/11/21 15:19:48 $ CERN Jean-Philippe Baud";
#endif /* not lint */

/*	lcg_gt - get the TURL for a given SURL and transfer protocol */
#include <errno.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include "gfal_api.h"
#include "lcg_util.h"
#include "gfal_dependencies.h"

int
lcg_gt (char *surl, char *protocol, char **turl, int *reqid, int *fileid, char **token)
{
	char *protoarray[3] = {protocol, "", NULL};
	return (lcg_gt2 (surl, protoarray, turl, reqid, fileid, token, NULL, 0, 0, NULL));
}

int
lcg_gtt (char *surl, char *protocol, char **turl, int *reqid, int *fileid, char **token, int timeout)
{
	char *protoarray[3] = {protocol, "", NULL};
        return (lcg_gt2 (surl, protoarray, turl, reqid, fileid, token, NULL, 0, timeout, NULL));
}

int
lcg_gtx (char *surl, char *protocol, char **turl, int *reqid, int *fileid,
        char **token, char *errbuf, int errbufsz)
{
	char *protoarray[3] = {protocol, "", NULL};
	return (lcg_gt2 (surl, protoarray, turl, reqid, fileid, token, errbuf, errbufsz, 0, NULL));
}

int
lcg_gtxt (char *surl, char *protocol, char **turl, int *reqid, int *fileid,
	char **token, char *errbuf, int errbufsz, int timeout)
{
	char *protoarray[3] = {protocol, "", NULL};
	return (lcg_gt2 (surl, protoarray, turl, reqid, fileid, token, errbuf, errbufsz, 0, NULL));
}

int
lcg_gt2 (char *surl, char **protocols, char **turl, int *reqid, int *fileid,
	char **token, char *errbuf, int errbufsz, int timeout, char *spacetokendesc)
{
	return (lcg_gt3 (surl, TYPE_NONE, TYPE_NONE, 0, protocols, turl, reqid, fileid, token,
				errbuf, errbufsz, timeout, spacetokendesc));
}

int
lcg_gt3 (char *surl, enum se_type defaulttype, enum se_type setype, int nobdii, char **protocols, char **turl,
		int *reqid, int *fileid, char **token, char *errbuf, int errbufsz, int timeout, char *spacetokendesc)
{
	return (lcg_gt4 (surl, defaulttype, setype, nobdii, protocols, turl, NULL, reqid, fileid, token,
				errbuf, errbufsz, timeout, spacetokendesc));
}

int
lcg_gt4 (char *surl, enum se_type defaulttype, enum se_type setype, int nobdii, char **protocols, char **turl,
		enum se_type *usedtype, int *reqid, int *fileid, char **token, char *errbuf, int errbufsz, int timeout,
		char *spacetokendesc)
{
	int n, rc;
	gfal_request req = NULL;
	gfal_internal gfal = NULL;
	gfal_filestatus *filestatuses = NULL;
	int *fileids = NULL;
    
    if (errbufsz > 0) {
        errbuf[0] = 0;
    }

	if (surl == NULL || protocols == NULL || turl == NULL ||
	    reqid == NULL || fileid == NULL || token == NULL) {
		errno = EFAULT;
		return (-1);
	}

	if (timeout > 0) {
		// for backward compatibility...
		gfal_set_timeout_connect (timeout);
		gfal_set_timeout_sendreceive (timeout);
		gfal_set_timeout_bdii (timeout);
		gfal_set_timeout_srm (timeout);
	}

	if ((req = gfal_request_new ()) == NULL)
		return (-1);
	req->nbfiles = 1;
	req->surls = (char **) &surl;
	req->defaultsetype = defaulttype;
	req->setype = setype;
	req->no_bdii_check = nobdii;
	req->protocols = protocols;
	req->srmv2_spacetokendesc = spacetokendesc;

	rc = lcg_gfal_functions.call_gfal_init (req, &gfal, errbuf, errbufsz);
	free (req);

	if (rc < 0)
		return (-1);

	if (lcg_gfal_functions.call_gfal_turlsfromsurls (gfal, errbuf, errbufsz) < 0) {
		gfal_internal_free (gfal);
		return (-1);
	}

    n = lcg_gfal_functions.call_gfal_get_results (gfal, &filestatuses);


	if ( n < 0 || filestatuses == NULL) {
		gfal_internal_free (gfal);
		gfal_errmsg (errbuf, errbufsz, GFAL_ERRLEVEL_ERROR, "Internal error");
		return (-1);
	}
	if (filestatuses[0].status != 0) {
		if (filestatuses[0].explanation)
			gfal_errmsg (errbuf, errbufsz, GFAL_ERRLEVEL_ERROR, "%s: %s",
					filestatuses[0].surl, filestatuses[0].explanation);
		else
			gfal_errmsg (errbuf, errbufsz, GFAL_ERRLEVEL_ERROR, "%s: %s",
					filestatuses[0].surl, strerror (filestatuses[0].status));

		errno = filestatuses[0].status;
		gfal_internal_free (gfal);
		return (-1);
	}

    if (filestatuses[0].turl == NULL) {
		gfal_internal_free (gfal);
		gfal_errmsg (errbuf, errbufsz, GFAL_ERRLEVEL_ERROR, "Server did not return a TURL!");
        errno = ENODATA;
		return (-1);
    }

	*turl = strdup (filestatuses[0].turl);

	rc = gfal_get_ids_setype (gfal, usedtype, reqid, &fileids, token);
	*fileid = fileids != NULL ? fileids[0] : -1;

	free (fileids);
	gfal_internal_free (gfal);
	return (rc > 0 ? 0 : -1);
}
