/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef lint
static char sccsid[] = "@(#)$RCSfile: lcg_lr.c,v $ $Revision: 1.21 $ $Date: 2008/11/10 12:38:16 $ CERN Jean-Philippe Baud";
#endif /* not lint */

/*	lcg_lr - get the list of replicas for a given LFN, GUID or SURL */
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include "gfal_api.h"
#include "lcg_util.h"

lcg_lr (char *file, char *vo, char *conf_file, int insecure, char ***pfns)
{
	return (lcg_lr2 (file, 0, vo, pfns, NULL, 0));
}

lcg_lrx (char *file, char *vo, char *conf_file, int insecure, char ***pfns,
		char *errbuf, int errbufsz)
{
	return (lcg_lr2 (file, 0, vo, pfns, errbuf, errbufsz));
}

lcg_lr2 (char *file, int nobdii, char *vo, char ***pfns, char *errbuf, int errbufsz)
{
	return (lcg_lr3 (file, nobdii, vo, pfns, 0, errbuf, errbufsz));
}

lcg_lr3 (char *file, int nobdii, char *vo, char ***pfns, int verbose, char *errbuf, int errbufsz)
{
	char *cat_type;
	char *actual_lfn = NULL, *actual_guid = NULL;
	char actual_file[GFAL_PATH_MAXLEN];
	char errmsg[GFAL_ERRMSG_LEN];

	if (file == NULL || pfns == NULL) {
		errno = EFAULT;
		return (-1);
	}

	gfal_set_verbose (verbose);

	if (vo && vo[0]) {
		if (strlen (vo) > GFAL_VO_MAXLEN) {
			gfal_errmsg (errbuf, errbufsz, GFAL_ERRLEVEL_ERROR, "%s: VO name too long", vo);
			errno = ENAMETOOLONG;
			return (-1);
		}
		if (gfal_set_vo (vo) < 0)
			return (-1);
	}

	if (canonical_url (file, NULL, actual_file, GFAL_PATH_MAXLEN, errbuf, GFAL_ERRMSG_LEN) < 0)
		return (-1);

	gfal_set_nobdii (nobdii);

	if (get_cat_type (&cat_type) < 0 )
		return (-1);
	if (strcmp (cat_type, "edg") && strcmp (cat_type, "lfc")) {
		free (cat_type);
		gfal_errmsg(errbuf, errbufsz, GFAL_ERRLEVEL_ERROR, "The catalog type is neither 'edg' nor 'lfc'.");
		errno = EINVAL;
		return (-1);
	}

	gfal_errmsg (NULL, 0, GFAL_ERRLEVEL_WARN, "Using grid catalog type: %s", cat_type);
	gfal_errmsg (NULL, 0, GFAL_ERRLEVEL_WARN, "Using grid catalog : %s", get_catalog_endpoint(errmsg, GFAL_ERRMSG_LEN));
	free (cat_type);

	if (strncmp (actual_file, "lfn:", 4) == 0) {
		actual_lfn = strdup (actual_file);
	} else if (strncmp (actual_file, "guid:", 5) == 0) {
		actual_guid = strdup (actual_file);
	} else if (strncmp (actual_file, "srm:", 4) == 0 || strncmp (actual_file, "sfn:", 4) == 0) {
		if ((actual_guid = gfal_guidforpfn (actual_file, errbuf, errbufsz)) == NULL)
			return (-1);
	} else {
		gfal_errmsg (errbuf, errbufsz, GFAL_ERRLEVEL_ERROR,
				"%s: File starts with neither \"lfn:\", \"guid:\", \"srm:\" nor \"sfn:\"", file);
		errno = EINVAL;
		return (-1);
	}

	*pfns = NULL;
	*pfns = gfal_get_replicas (actual_lfn, actual_guid, errbuf, errbufsz);

	if (actual_lfn) free (actual_lfn);
	if (actual_guid) free (actual_guid);
	return (*pfns ? 0 : -1);
}
