/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef lint
static char sccsid[] = "@(#)$RCSfile: lcg_rf.c,v $ $Revision: 1.32 $ $Date: 2009/05/19 12:54:39 $ CERN Jean-Philippe Baud";
#endif /* not lint */

/*	lcg_rf - registers in the LRC (and optionally in the RMC) a file
 *	residing on an SE
 */
#include <errno.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <sys/stat.h>
#include <unistd.h>
#include <uuid/uuid.h>
#include "gfal_api.h"
#include "lcg_util.h"
#include "gfal_dependencies.h"
#include "globus_dependencies.h"

lcg_rf (char *surl, char *guid, char *lfn, char *vo, char *conf_file,
	int insecure, int verbose, char *actual_guid)
{
	return (lcg_rf3 (surl, guid, lfn, TYPE_NONE, TYPE_NONE, 0, vo, conf_file, insecure, verbose, 
		0, actual_guid, NULL, 0));
}


lcg_rft (char *surl, char *guid, char *lfn, char *vo, char *conf_file,
	int insecure, int verbose, int timeout, char *actual_guid)
{
	return (lcg_rf3 (surl, guid, lfn, TYPE_NONE, TYPE_NONE, 0, vo, conf_file, insecure, verbose, 
		timeout, actual_guid, NULL, 0));
}

lcg_rfx (char *surl, char *guid, char *lfn, char *vo, char *conf_file,
	int insecure, int verbose, char *actual_guid, char *errbuf, int errbufsz)
{
	return (lcg_rfxt (surl, guid, lfn, vo, conf_file, insecure, verbose, 
		0,  actual_guid, errbuf, errbufsz));
	return (lcg_rf3 (surl, guid, lfn, TYPE_NONE, TYPE_NONE, 0, vo, conf_file, insecure, verbose, 
		0, actual_guid, errbuf, errbufsz));
}

lcg_rfxt (char *surl, char *guid, char *lfn, char *vo, char *conf_file,
	int insecure, int verbose, int timeout, char *actual_guid, char *errbuf, int errbufsz)
{
	return (lcg_rf3 (surl, guid, lfn, TYPE_NONE, TYPE_NONE, 0, vo, conf_file, insecure, verbose, 
		timeout, actual_guid, errbuf, errbufsz));
}

lcg_rf3 (char *surl, char *guid, char *lfn, enum se_type defaulttype, enum se_type setype,
		int nobdii, char *vo, char *conf_file, int insecure, int verbose, int timeout,
		char *actual_guid, char *errbuf, int errbufsz)
{
	off64_t filesize;
	char *cat_type;
	char cattype[8];
	char actual_lfn[1024] = "";
	char *generated_guid = NULL;
	gfal_request req;
	gfal_internal gfal;
	gfal_filestatus *filestatuses;
	char tmpbuf[21];

	gfal_set_verbose (verbose);

	if (actual_guid)
		*actual_guid = '\0';
	if (surl == NULL) {
		errno = EFAULT;
		return (-1);
	}

	if (timeout > 0) {
		// for backward compatibility...
		gfal_set_timeout_connect (timeout);
		gfal_set_timeout_sendreceive (timeout);
		gfal_set_timeout_bdii (timeout);
		gfal_set_timeout_srm (timeout);
	}

	if (strncmp (surl, "srm:/", 5) && strncmp (surl, "sfn:/", 5)) {
		gfal_errmsg (errbuf, errbufsz, GFAL_ERRLEVEL_ERROR, "Source URL starts with neither 'srm:/' nor 'sfn:/'.");
		errno = EINVAL;
		return (-1);
	}
	if (vo && vo[0]) {
		if (strlen (vo) > GFAL_VO_MAXLEN) {
			gfal_errmsg (errbuf, errbufsz, GFAL_ERRLEVEL_ERROR, "%s: VO name too long", vo);
			errno = ENAMETOOLONG;
			return (-1);
		}
		if (gfal_set_vo (vo) < 0)
			return (-1);
	}

	gfal_set_nobdii (nobdii);

	if (get_cat_type (&cat_type) < 0 ) {
		return (-1);
	}
	if (strcmp (cat_type, "edg") && strcmp (cat_type, "lfc")) {
		free (cat_type);
		gfal_errmsg (errbuf, errbufsz, GFAL_ERRLEVEL_ERROR, "The catalog type is neither 'edg' nor 'lfc'.");
		errno = EINVAL;
		return (-1);
	}
	strcpy (cattype, cat_type);
	free (cat_type);

	if (lfn && lfn[0]) {
		if (canonical_url (lfn, "lfn", actual_lfn, GFAL_PATH_MAXLEN, errbuf, GFAL_ERRMSG_LEN) < 0)
			return (-1);
	}

	/* get file size */

	if ((req = gfal_request_new ()) == NULL)
		return (-1);
	req->nbfiles = 1;
	req->surls = &surl;
	req->defaultsetype = defaulttype;
	req->setype = setype;
	req->no_bdii_check = nobdii;

	if (lcg_gfal_functions.call_gfal_init (req, &gfal, errbuf, errbufsz) < 0) {
		free (req);
		return (-1);
	}
	free (req);

	if (lcg_gfal_functions.call_gfal_ls (gfal, errbuf, errbufsz) < 0) {
		gfal_internal_free (gfal);
		return (-1);
	}
	if (lcg_gfal_functions.call_gfal_get_results (gfal, &filestatuses) < 0 || filestatuses == NULL) {
		gfal_internal_free (gfal);
		gfal_errmsg (errbuf, errbufsz, GFAL_ERRLEVEL_ERROR, "Internal error");
		return (-1);
	}
	if (filestatuses[0].status != 0) {
		gfal_errmsg (errbuf, errbufsz, GFAL_ERRLEVEL_ERROR, "%s: %s",
				filestatuses[0].surl, strerror (filestatuses[0].status));
		errno = filestatuses[0].status;
		gfal_internal_free (gfal);
		return (-1);
	}
	filesize = filestatuses[0].stat.st_size;
	gfal_internal_free (gfal);

	gfal_errmsg (NULL, 0, GFAL_ERRLEVEL_WARN, "Using grid catalog type: %s", cattype);
	gfal_errmsg (NULL, 0, GFAL_ERRLEVEL_WARN, "Using grid catalog : %s", get_catalog_endpoint(errbuf, errbufsz));
	gfal_errmsg (NULL, 0, GFAL_ERRLEVEL_WARN, "Site URL to be registered: %s", surl);
	sprintf (tmpbuf, GFAL_LONG64_FORMAT, filesize);
	gfal_errmsg (NULL, 0, GFAL_ERRLEVEL_WARN, "File size: %s", tmpbuf);

	if (guid == NULL && (guid = generated_guid = gfal_generate_guid (errbuf, errbufsz)) == NULL)
		return (-1);

	if (lcg_gfal_functions.call_gfal_register_file (actual_lfn, guid, surl, 0, filesize, 0, errbuf, errbufsz) < 0) {
		if (generated_guid) free (generated_guid);
		return (-1);
	}

	if (actual_guid && guid)
		strcpy (actual_guid, guid);

	if (generated_guid) free (generated_guid);
	return (0);
}
