/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef lint
static char sccsid[] = "@(#)$RCSfile: lcg_sd.c,v $ $Revision: 1.12 $ $Date: 2008/11/21 15:19:48 $ CERN Jean-Philippe Baud";
#endif /* not lint */

/*	lcg_sd - set file status to "Done" */
#include <errno.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "gfal_api.h"
#include "lcg_util.h"

lcg_sd (char *surl, int reqid, int fileid, char *token, int oflag)
{
	return (lcg_sdx (surl, reqid, fileid, token, oflag, NULL, 0));
}

lcg_sdt (char *surl, int reqid, int fileid, char *token, int oflag, int timeout)
{
        return (lcg_sdxt (surl, reqid, fileid, token, oflag, NULL, 0, timeout));
}

lcg_sdx (char *surl, int reqid, int fileid, char *token, int oflag,
        char *errbuf, int errbufsz)
{
	return (lcg_sdxt (surl, reqid, fileid, token, oflag, errbuf, errbufsz, 0));
}

lcg_sdxt (char *surl, int reqid, int fileid, char *token, int oflag,
	char *errbuf, int errbufsz, int timeout)
{
	return (lcg_sd3 (surl, 0, reqid, fileid, token, errbuf, errbufsz, timeout));
}

lcg_sd3 (char *surl, int nobdii, int reqid, int fileid,
		char *reqtoken, char *errbuf, int errbufsz, int timeout)
{
	int rc, n;
	gfal_request req;
	gfal_internal gfal;
	gfal_filestatus *filestatuses;

	if (timeout > 0) {
		// for backward compatibility...
		gfal_set_timeout_connect (timeout);
		gfal_set_timeout_sendreceive (timeout);
		gfal_set_timeout_bdii (timeout);
		gfal_set_timeout_srm (timeout);
	}

	if ((req = gfal_request_new ()) == NULL)
		return (-1);
	req->nbfiles = 1;
	req->surls = (char **) &surl;
	req->no_bdii_check = nobdii;

	if (reqtoken)
		req->setype = TYPE_SRMv2;
	else
		req->setype = TYPE_SRM;

	rc = gfal_init (req, &gfal, errbuf, errbufsz);
	free (req);

	if (rc < 0)
		return (-1);

	if (reqtoken)
		rc = gfal_set_ids (gfal, 0, NULL, 0, reqtoken, errbuf, errbufsz);
	else
		rc = gfal_set_ids (gfal, 1, &fileid, reqid, NULL, errbuf, errbufsz);

	if (rc < 0) {
		gfal_internal_free (gfal);
		return (-1);
	}

	if (gfal_set_xfer_done (gfal, errbuf, errbufsz) < 0) {
		gfal_internal_free (gfal);
		return (-1);
	}

	if ((n = gfal_get_results (gfal, &filestatuses)) < 0 || filestatuses == NULL) {
		gfal_internal_free (gfal);
		gfal_errmsg (errbuf, errbufsz, GFAL_ERRLEVEL_ERROR, "Internal error");
		return (-1);
	}
	if (filestatuses[0].status != 0) {
		if (filestatuses[0].explanation)
			gfal_errmsg (errbuf, errbufsz, GFAL_ERRLEVEL_ERROR, "%s: %s", filestatuses[0].surl, filestatuses[0].explanation);
		else
			gfal_errmsg (errbuf, errbufsz, GFAL_ERRLEVEL_ERROR, "%s: %s", filestatuses[0].surl, strerror (filestatuses[0].status));

		gfal_internal_free (gfal);
		errno = filestatuses[0].status;
		return (-1);
	}

	gfal_internal_free (gfal);
	errno = 0;
	return (0);
}
